{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....: Delphi IDE Extension Expert
 * File.......: ExpReg.pas
 * Autor......: Anton A. Mints
 * Date.......: 10 June 1997
 * Description: Class to manage worh with Windows Registry settings
 *
 * $Archive: /ide expert/classes/expreg.pas $
 *
 * $History: expreg.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/classes
 *
 *************}

unit ExpReg;

interface

uses
  Registry, Menus, Graphics;

type
 { TExpRegistry }
  TExpRegistry = class (TRegistry)
  private
    procedure SetShortCut (Index: Integer; Value: TShortCut);
    function GetShortCut (Index: Integer): TShortCut;

    procedure SetAuthor (Value: String);
    function GetAuthor: String;

    procedure SetAuthorTAG (Value: String);
    function GetAuthorTAG: String;

    procedure SetColor (Index: Integer; Value: TColor);
    function GetColor (Index: Integer): TColor;

  public
    constructor Create;

    property InsHeaderSCut: TShortCut index 1 read GetShortCut write SetShortCut;
    property InsSeparSCut: TShortCut index 2 read GetShortCut write SetShortCut;
    property BrowseTagSCut: TShortCut index 3 read GetShortCut write SetShortCut;
    property ApplyTemplSCut: TShortCut index 4 read GetShortCut write SetShortCut;

    property Author: String read GetAuthor write SetAuthor;
    property AuthorTAG: String read GetAuthorTAG write SetAuthorTAG;

    property EventColor: TColor index 1 read GetColor write SetColor;
    property ProtectedColor: TColor index 2 read GetColor write SetColor;
    property PrivateColor: TColor index 3 read GetColor write SetColor;
    property PublicColor: TColor index 4 read GetColor write SetColor;
    property PublishedColor: TColor index 5 read GetColor write SetColor;

  end;


implementation

uses
  Windows, Classes, SysUtils;

{*******************************************************************}
{*                     CLASS TExpRegistry                          *}
{*******************************************************************}

constructor TExpRegistry.Create;
begin
  inherited Create;

  // Personal information
  RootKey := HKEY_USERS;
end;

{*************
* Name.......: SetShortCut
*}
procedure TExpRegistry.SetShortCut (Index: Integer; Value: TShortCut);
begin
  if OpenKey ('.Default\Software\CEO\Key Templates', True ) then
  begin
    WriteString('ShortCut' + IntToStr (Index), ShortCutToText(Value));
    CloseKey;
  end;
end;


{*************
* Name.......: GetShortCut
*}
function TExpRegistry.GetShortCut (Index: Integer): TShortCut;
begin
  // Init with default value
  Result := Menus.ShortCut (0, []);

  if OpenKey ('.Default\Software\CEO\Key Templates', False) and
      ValueExists ('ShortCut' + IntToStr (Index)) then
  begin
    Result := TextToShortCut (ReadString ('ShortCut' + IntToStr (Index)));
    CloseKey;
  end;
end;


{*************
* Name.......: SetAuthor
*}
procedure TExpRegistry.SetAuthor (Value: String);
begin
  if OpenKey ('.Default\Software\CEO\Key Templates', True ) then
  begin
    WriteString('Author', Value);
    CloseKey;
  end;
end;


{*************
* Name.......: GetAuthor
*}
function TExpRegistry.GetAuthor: String;
begin
  // Init with default value
  Result := '';

  if OpenKey ('.Default\Software\CEO\Key Templates', False) and
      ValueExists ('Author') then
  begin
    Result := ReadString ('Author');
    CloseKey;
  end;
end;


{*************
* Name.......: SetAuthorTAG
*}
procedure TExpRegistry.SetAuthorTAG (Value: String);
begin
  if OpenKey ('.Default\Software\CEO\Key Templates', True ) then
  begin
    WriteString('AuthorTAG', Value);
    CloseKey;
  end;
end;


{*************
* Name.......: GetAuthorTAG
*}
function TExpRegistry.GetAuthorTAG: String;
begin
  // Init with default value
  Result := '';

  if OpenKey ('.Default\Software\CEO\Key Templates', False) and
      ValueExists ('AuthorTAG') then
  begin
    Result := ReadString ('AuthorTAG');
    CloseKey;
  end;
end;


{*************
* Name.......: SetColor
*}
procedure TExpRegistry.SetColor (Index: Integer; Value: TColor);
begin
  if OpenKey ('.Default\Software\CEO\Key Templates', True) then
  begin
    WriteInteger ('Color' + IntToStr (Index), Value);
    CloseKey;
  end;
end;


{*************
* Name.......: GetColor
*}
function TExpRegistry.GetColor (Index: Integer): TColor;
begin
  // Init with default value
  Result := clWhite;

  if OpenKey ('.Default\Software\CEO\Key Templates', False) and
      ValueExists ('Color' + IntToStr (Index)) then
  begin
    Result := ReadInteger ('Color' + IntToStr (Index));
    CloseKey;
  end;
end;

end.
