{*************
 *
 * Copyright (c) 1996 CEO Corporation
 *
 * Project....:
 * File.......: TmplList.pas
 * Autor......: Anton A. Mitns
 * Date.......: 03 December 1996
 * Description: List to keep template's tokens in memory
 *
 * $Archive: /ide expert/classes/tmpllist.pas $
 *
 * $History: tmpllist.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/classes
 *
 *************}

unit TmplList;

interface

{$H+}

uses
  Classes;

type
  TTmplKey = String;
  TTmplTxt = String;

{ TTemplateItemRec }
  TTemplateItemRec = record
    TmplKey: TTmplKey;
    TmplTxt: TTmplTxt;
  end;

{ TTemplateItem }
  TTemplateItem = class
    TmplKey: TTmplKey;
    TmplTxt: TTmplTxt;
  end;

type
  TTemplateList = class (TList)
  private
    function GetItem (Index: Integer): TTemplateItemRec;

  public
    destructor Destroy; override;

    procedure Clear;
    function  Add (Item: TTemplateItemRec): Integer;
    procedure Insert (Index: Integer; Item: TTemplateItemRec);
    procedure Delete(Index: Integer);
    // Find by CheckNum
    function IndexOf (Item: TTmplKey): Integer;

    property Items[Index: Integer]: TTemplateItemRec read GetItem; default;
  end;

implementation

{*******************************************************************}
{*                   CLASS TTransactionGuide                       *}
{*******************************************************************}

{*************
* Name.......: Destroy
*}
destructor TTemplateList.Destroy;
begin
  // Clear all list Conents
  Clear;
  inherited Destroy;
end;


{*************
* Name.......: Clear
*}
procedure TTemplateList.Clear;
var
  I: Integer;
begin
  // Free all items stored in the list
  if Count <= 0 then Exit;
  for I := 0 to Count - 1 do
    TTemplateItem(inherited Items[I]).Free;
  // perform inherited clearing
  inherited Clear;
end;


{*************
* Name.......: GetItem
*}
function TTemplateList.GetItem (Index: Integer): TTemplateItemRec;
begin
  with TTemplateItem(inherited Items[Index]) do
  begin
    Result.TmplKey := TmplKey;
    Result.TmplTxt := TmplTxt;
  end;
end;


{*************
* Name.......: Add
*}
function TTemplateList.Add (Item: TTemplateItemRec): Integer;
var
  TmplItem: TTemplateItem;
begin
  // create the class same type as TTemplateItemRec
  TmplItem := TTemplateItem.Create;
  // fill it's fields
  TmplItem.TmplKey := Item.TmplKey;
  TmplItem.TmplTxt := Item.TmplTxt;
  // and add it as maked in parent class
  Result := inherited Add (TmplItem);
end;


{*************
* Name.......: insert
*}
procedure TTemplateList.Insert (Index: Integer; Item: TTemplateItemRec);
var
  TmplItem: TTemplateItem;
begin
  // create the class same type as TTemplateItemRec
  TmplItem := TTemplateItem.Create;
  // fill the fields
  TmplItem.TmplKey := Item.TmplKey;
  TmplItem.TmplTxt := Item.TmplTxt;
  // and do as papa do
  inherited Insert (Index, TmplItem);
end;


{*************
* Name.......: Delete
*}
procedure TTemplateList.Delete(Index: Integer);
begin
  TTemplateItem(inherited Items[Index]).Free;
  inherited Delete (Index);
end;


{*************
* Name.......: IndexOf
*}
function TTemplateList.IndexOf (Item: TTmplKey): Integer;
var
  I: Integer;
begin
  Result := -1;
  if Count <= 0 then Exit;
  for I := 0 to Count - 1 do
  begin
    if ( TTemplateItem(inherited Items[I]).TmplKey = Item ) then
    begin
      Result := I;
      Break;
    end;
  end;
end;

end.
