{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....:
 * File.......: ExpSet.pas
 * Autor......: 16711680
 * Date.......: 10 June 1997
 * Description: Form to manage expert settings
 *
 * $Archive: /ide expert/forms/expset.pas $
 *
 * $History: expset.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/forms
 *
 *************}

unit ExpSet;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, ExtCtrls, DB, DBTables, DBCtrls, Grids,
  DBGrids;

type
  TfrmExpertSettings = class(TForm)
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    pgSettings: TPageControl;
    tshtGlobal: TTabSheet;
    tshtTags: TTabSheet;
    gbtagBrowserColors: TGroupBox;
    btnPrivate: TButton;
    btnProtected: TButton;
    btnEvent: TButton;
    btnPublished: TButton;
    pnlEvent: TPanel;
    pnlProtected: TPanel;
    pnlPrivate: TPanel;
    pnlPublished: TPanel;
    pnlPublic: TPanel;
    btnPublic: TButton;
    dlgColor: TColorDialog;
    gbShortCuts: TGroupBox;
    hkInsHeader: THotKey;
    hkApplyTemplate: THotKey;
    hkBrowseTags: THotKey;
    hkInsSeparator: THotKey;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    gbAuthor: TGroupBox;
    edtAuthor: TEdit;
    Label2: TLabel;
    edtAuthorTAG: TEdit;
    lblAuthorTag: TLabel;
    tshtTemplates: TTabSheet;
    grdKeyStrings: TDBGrid;
    Label6: TLabel;
    Label7: TLabel;
    dbmemTemplate: TDBMemo;
    tblKey_Exp: TTable;
    DataSource: TDataSource;

    procedure SetColorClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);

  public
    constructor Create (AOwner: TComponent); override;

    function ShowModal (TmplDirectory: String): Integer;
  end;

implementation

uses
  ExpReg, AExpert;

{$R *.DFM}

{*******************************************************************}
{*                   CLASS TFrmExpertSettings                      *}
{*******************************************************************}

constructor TFrmExpertSettings.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);

  // First page is active
  pgSettings.ActivePage := tshtGlobal;
end;


{*************
* Name.......: ShowModal
* Description: Initialize controls, show form and save changes
*             after form is closed
*}
function TFrmExpertSettings.ShowModal (TmplDirectory: String): Integer;
var
  ExpRegistry: TExpRegistry;

begin
  ExpRegistry := TExpRegistry.Create;
  try
    tblKey_Exp.DatabaseName := TmplDirectory;
    tblKey_Exp.Open;

    hkInsHeader.HotKey := ExpRegistry.InsHeaderSCut;
    hkInsSeparator.HotKey := ExpRegistry.InsSeparSCut;
    hkBrowseTags.HotKey := ExpRegistry.BrowseTagSCut;
    hkApplyTemplate.HotKey := ExpRegistry.ApplyTemplSCut;

    edtAuthor.Text := ExpRegistry.Author;
    edtAuthorTAG.Text := ExpRegistry.AuthorTAG;

    pnlEvent.Color := ExpRegistry.EventColor;
    pnlPublished.Color := ExpRegistry.PublishedColor;
    pnlPublic.Color := ExpRegistry.PublicColor;
    pnlPrivate.Color := ExpRegistry.PrivateColor;
    pnlProtected.Color := ExpRegistry.ProtectedColor;

    Result := inherited ShowModal;

    if ( Result = mrOk ) then
      begin
        ExpRegistry.Author := edtAuthor.Text;
        ExpRegistry.AuthorTAG := edtAuthorTAG.Text;

        ExpRegistry.InsHeaderSCut := hkInsHeader.HotKey;
        ExpRegistry.InsSeparSCut := hkInsSeparator.HotKey;
        ExpRegistry.BrowseTagSCut := hkBrowseTags.HotKey;
        ExpRegistry.ApplyTemplSCut := hkApplyTemplate.HotKey;

        ExpRegistry.EventColor := pnlEvent.Color;
        ExpRegistry.PublishedColor := pnlPublished.Color;
        ExpRegistry.PublicColor := pnlPublic.Color;
        ExpRegistry.PrivateColor := pnlPrivate.Color;
        ExpRegistry.ProtectedColor := pnlProtected.Color;

        tblKey_Exp.ApplyUpdates;
      end
    else
      begin
        tblKey_Exp.CancelUpdates;
      end;

  finally
    tblKey_Exp.Close;
    ExpRegistry.Free;
  end;
end;


{*************
* Name.......: SetColor (OnCLick EVENT)
* Description: Change color on the panel according with returned color
*}
procedure TfrmExpertSettings.SetColorClick(Sender: TObject);
var
  Name: string;

begin
  if dlgColor.Execute then
  begin
    Name := (Sender as TButton).Name;

    // detect what option button has been pressed
    // and perform color dialog for corresponding option.
    if Name = 'btnEvent' then pnlEvent.Color := dlgColor.Color
    else if Name = 'btnProtected' then pnlProtected.Color := dlgColor.Color
    else if Name = 'btnPrivate' then pnlPrivate.Color := dlgColor.Color
    else if Name = 'btnPublic' then pnlPublic.Color := dlgColor.Color
    else if Name = 'btnPublished' then pnlPublished.Color := dlgColor.Color;
  end;
end;


{*************
* Name.......: The Form (OnCloseQuery EVENT)
*}
procedure TfrmExpertSettings.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  // Perform post explisity
  if tblKey_Exp.State in [dsEdit, dsInsert] then
    tblKey_Exp.Post;
end;

end.

