{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....:
 * File.......: TagBrws.pas
 * Autor......:
 * Date.......: 06 June 1997
 * Description: Simple form to browse the Tag list.
 *
 * $Archive: /ide expert/forms/tagbrws.pas $
 *
 * $History: tagbrws.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/forms
 *
 *************}

unit TagBrws;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, TagLst, EditIntf, RXCtrls;

type
  TReturnPosType = (rptDeclaration, rptImplementation, rptReference);

  TFrmTagBrowser = class(TForm)
    lbxTags: TListBox;
    pnlSearch: TPanel;
    lbxReferences: TTextListBox;
    procedure lbxTagsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lbxTagsDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lbxTagsMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    procedure lbxTagsKeyPress(Sender: TObject; var Key: Char);
    procedure FormActivate(Sender: TObject);
    procedure lbxReferencesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lbxTagsClick(Sender: TObject);

  private
    FFileName: String;
    FScrolLock: Boolean;
    FEditorInterface: TIEditorInterface;

    EventColor, PublishedColor,
    PublicColor, PrivateColor,
    ProtectedColor: TColor;

    lstTags,
    lstImplementPos,
    lstReferences: TTagList;
    ReturnPosType: TReturnPosType;

    procedure ShowReferenceList;
    procedure HideReferenceList;

    function GetImplLineNum: Integer;
    function GetDeclLineNum: Integer;
    function GetRefLineNum: Integer;

  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;

    function ShowModal (EditorInterface: TIEditorInterface;
      FileName: String; lstTags, lstImplementPos: TTagList;
      InitSelectWord: String): Integer;
  end;

implementation

uses
  TagPrsr, RefPrsr, Tokens, ExpReg, EdReader;

var
  LastInitSelectWord: String;

{$R *.DFM}

{*******************************************************************}
{*                    CLASS TFrmTagBrowser                         *}
{*******************************************************************}

constructor TFrmTagBrowser.Create (AOwner: TComponent);
var
  ExpRegistry: TExpRegistry;
begin
  inherited Create (AOwner);

  ExpRegistry := TExpRegistry.Create;
  try
    EventColor := ExpRegistry.EventColor;
    PublishedColor := ExpRegistry.PublishedColor;
    PublicColor := ExpRegistry.PublicColor;
    PrivateColor := ExpRegistry.PrivateColor;
    ProtectedColor := ExpRegistry.ProtectedColor;
  finally
    ExpRegistry.Free;
  end;

  lstReferences := TTagList.Create;

  // Assign default value
  FEditorInterface := nil;
  FScrolLock := True; 
end;


destructor TFrmTagBrowser.Destroy;
begin
  lstReferences.Free;
  inherited Destroy;
end;


{*************
* Name.......: ShowReferenceList
*}
procedure TFrmTagBrowser.ShowReferenceList;
var
  SubString: String;
  AlreadyVisible: Boolean;
  I: Integer;

begin
  AlreadyVisible := lbxReferences.Visible;

  // Show the list of refenreces only for the
  // procedures, functions, constructors or
  //destructors.
  if not (lstTags[lbxTags.ItemIndex].tagType in [usProcedure,
    usFunction, usConstructor, usDestructor]) then Exit;

  // Getting substring
  Substring := Token (lstTags[lbxTags.ItemIndex].tagName, 1);

  // Looking for the reference list
  lstReferences.Clear;
  FindReference (FFileName, Substring,
    lstReferences, lstImplementPos);
  lbxReferences.Visible := True;

  // Full fill the list box
  lbxReferences.Clear;
  for I := 0 to lstReferences.Count - 1 do
    lbxReferences.Items.Add (lstReferences[I].tagName);

  // If the list isn't empty select first item
  if ( lstReferences.Count > 0 ) then
    lbxReferences.ItemIndex := 0;

  //...and move in focus
  if not AlreadyVisible and ( lstReferences.Count > 0 ) then
    lbxReferences.SetFocus;

end;


procedure TFrmTagBrowser.HideReferenceList;
begin
  lbxReferences.Visible := False;
  lstReferences.Clear;
  // Set focus to the tag listbox any ways
  lbxTags.SetFocus;
end;


{*************
* Name.......: GetImplLineNum
*}
function TFrmTagBrowser.GetImplLineNum: Integer;
var
  TagName: String;
  ImplPos: Integer;

begin
  Result := -1;

  // Only procedure or funciton can have implementation pos
  if not (lstTags[lbxTags.ItemIndex].tagType in [usProcedure,
    usFunction, usConstructor, usDestructor]) then Exit;

  // Extract procedure or function name now
  TagName := Token (lstTags[lbxTags.ItemIndex].tagName, 1);

  // Looking for implementation pos in the list
  ImplPos := FindInTagList (lstImplementPos, TagName, 0);

  // If line was found, extract line number
  if ( ImplPos >= 0 ) then
    Result := lstImplementPos[ImplPos].LineNum;
end;


{*************
* Name.......: GetDeclLineNum
*}
function TFrmTagBrowser.GetDeclLineNum: Integer;
begin
  Result := -1;

  // Nothing selected in the list
  if ( lbxTags.ItemIndex < 0 ) then Exit;

  try
    // Return declaration position
    Result := lstTags[lbxTags.ItemIndex].LineNum;
  except
    Result := -1;
  end;
end;


{*************
* Name.......: GetRefLineNum
*}
function TFrmTagBrowser.GetRefLineNum: Integer;
begin
  Result := -1;

  if ( lbxReferences.ItemIndex < 0 ) then Exit;

  try
    Result := lstReferences[lbxReferences.ItemIndex].LineNum;
  except
  end;
end;


{*************
* Name.......: ShowModal
*}
function TFrmTagBrowser.ShowModal (EditorInterface: TIEditorInterface;
  FileName: String; lstTags, lstImplementPos: TTagList;
  InitSelectWord: String): Integer;
var
  I: Integer;
  ImplementPos: Integer;
  TagName: String;

begin
  // Use internal variables
  try
    FFileName := FileName;
    FEditorInterface := EditorInterface;
    Self.lstTags := lstTags;
    Self.lstImplementPos := lstImplementPos;

    // Assume an error
    Result := -1;

    // Exit if there are nothing to select
    // if ( lstTags.Count <= 0 ) then Exit;

    // Assign Tads to the Listbox
    lbxTags.Clear;
    for I := 0 to lstTags.Count - 1 do
      lbxTags.Items.Add (lstTags[I].tagName);

    // Initialize search box
    lbxTags.ItemIndex := FindInTagList (lstTags, InitSelectWord, 0);

    // If nothing was selected, select first item explisity
    if ( lbxTags.ItemIndex < 0 ) then
      lbxTags.ItemIndex := 0;

    // Show the form.
    inherited ShowModal;

    if (ModalResult = mrOk) and (lbxTags.ItemIndex >= 0) then
    try
      if ( ReturnPosType = rptImplementation ) then
        Result := GetImplLineNum
      else if ( ReturnPosType = rptReference ) then
        Result := GetRefLineNum;

      // Return declaration position
      if ( Result <= 0 ) or ( ReturnPosType = rptDeclaration ) then
        Result := GetDeclLineNum;

      // Keep last selected position if somthing was selected
      if ( pnlSearch.Caption <> '' ) then
        LastInitSelectWord := pnlSearch.Caption
      else
        // Extract procedure or function name now
        LastInitSelectWord := Token (lstTags[lbxTags.ItemIndex].tagName, 1);

    except
    end;

  finally
    lstTags := nil;
    lstImplementPos := nil;
    FEditorInterface := nil;
  end;
end;


{*************
* Name.......: lbxTags (OnMeasureItem EVENT)
*}
procedure TFrmTagBrowser.lbxTagsMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
begin
  // Class and Implementation make larger.
  if ( lstTags[Index].tagType = usClass ) then
    Height := Trunc (Height * 2);
end;


{*************
* Name.......: lbxTags (OnDrawItem EVENT)
*}
procedure TFrmTagBrowser.lbxTagsDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Offset: Integer;
  TopOff: Integer;

  lineTxt: String;
  lineLen: Integer;

begin
  with lbxTags, lbxTags.Canvas do
  begin
    // default provide brush
    if ( odSelected in State ) then
      Brush.Color := clHighlight
    else
      // Change color if item isn't selecte
      if not ( odSelected in State ) then
      try
        case lstTags[Index].tagSubType of
          usEvents:     Brush.Color := EventColor;
          usPrivate:    Brush.Color := PrivateColor;
          usProtected:  Brush.Color := ProtectedColor;
          usPublic:     Brush.Color := PublicColor;
          usPublished:  Brush.Color := PublishedColor;
        end;
      except
        Brush.Color := clWindow;
      end;

    // Adjust for class and implementation lines
    if ( lstTags[Index].tagType = usClass ) then
      begin
        TopOff := 8;
        Font.Style := Font.Style + [fsBold]
      end
    else
      begin
        TopOff := 0;
        Font.Style := Font.Style - [fsBold]
      end;

    case ( lstTags[Index].tagType ) of
      usProcedure:    lineTxt := 'proc';
      usFunction:     lineTxt := 'func';
      usProperty:     lineTxt := 'prop';
      usDestructor:   lineTxt := 'DECT';
      usConstructor:  lineTxt := 'CONS';
    else
      lineTxt := '';
    end;

    FillRect(Rect);	// clear the rectangle
    Offset := 2; // provide default offset

    lineTxt := lineTxt + ' ' + lstTags[Index].tagName;
    lineLen := Length (lineTxt);

    Rect.Left := Rect.Left + Offset;
    Rect.Top  := Rect.Top + TopOff;

    SetBkMode (Canvas.Handle, TRANSPARENT);
    TextOut (Rect.Left, Rect.Top, lineTxt);
  end;
end;


{*************
* Name.......: lbxTags (OnKeyDown EVENT)
*}
procedure TFrmTagBrowser.lbxTagsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_DIVIDE: begin
      FScrolLock := not FScrolLock;
    end;

    VK_ADD: begin
      ShowReferenceList;
    end;

    VK_SUBTRACT: begin
      HideReferenceList;
    end;

    VK_RETURN: begin
      ModalResult := mrOk;
      ReturnPosType := rptImplementation;
    end;

    VK_MULTIPLY: begin
      pnlSearch.Caption := LastInitSelectWord;
      lbxTags.ItemIndex := FindInTagList (lstTags,
        pnlSearch.Caption, 0);
    end;

    VK_SPACE: begin
      ModalResult := mrOk;
      ReturnPosType := rptDeclaration;
    end;

    VK_DELETE, VK_BACK: begin
      pnlSearch.Caption := Copy (pnlSearch.Caption, 1, Length (pnlSearch.Caption) - 1);
      lbxTags.ItemIndex := FindInTagList (lstTags, pnlSearch.Caption, 0);
    end;

    VK_UP, VK_DOWN, VK_HOME, VK_END, VK_LEFT, VK_RIGHT: begin
      pnlSearch.Caption := '';
    end;

    VK_F3: begin
      lbxTags.ItemIndex := FindInTagList (lstTags,
        pnlSearch.Caption, lbxTags.ItemIndex + 1);
    end;

    27: begin
      Close;
    end;
  end;

  if Length (pnlSearch.Caption) <= 0 then
    pnlSearch.Hide
  else
    pnlSearch.Show;
end;


{*************
* Name.......: lbxTags (OnKeyPress EVENT)
*}
procedure TFrmTagBrowser.lbxTagsKeyPress(Sender: TObject; var Key: Char);
begin
  if IsCharAlpha (Key) then
  begin
    pnlSearch.Caption := pnlSearch.Caption + Key;
    pnlSearch.Show;
    lbxTags.ItemIndex := FindInTagList (lstTags, pnlSearch.Caption, lbxTags.ItemIndex);
    Key := ' ';
  end;
end;


{*************
* Name.......: lbxReferences (OnKeyDown EVENT)
*}
procedure TFrmTagBrowser.lbxReferencesKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_SUBTRACT: begin
      HideReferenceList;
    end;

    VK_RETURN: begin
      ModalResult := mrOk;
      ReturnPosType := rptReference;
    end;

    VK_ESCAPE: begin
      HideReferenceList;
    end;

    VK_SPACE: begin
      ModalResult := mrOk;
      ReturnPosType := rptDeclaration;
    end;
  end;
end;


{*************
* Name.......: The Form (OnActivate EVENT)
* Description: Move form to the proper place.
*}
procedure TFrmTagBrowser.FormActivate(Sender: TObject);
begin
  Height := Screen.Height - 20;
  Top := 10;
  Left := Screen.Width - Width - 10;
end;


procedure TFrmTagBrowser.lbxTagsClick(Sender: TObject);
var
  TopPos: TEditPos;

begin
  if ( FScrolLock ) then
  begin
    if ( Sender = lbxTags ) then
      TopPos.Line := GetImplLineNum
    else
      TopPos.Line := GetRefLineNum;

    TopPos.Col  := 1;

    // Scroll view to the selected line
    if ( TopPos.Line > 0 ) and
        Assigned (FEditorInterface) then
      IEISetTopPos (FEditorInterface, TopPos);
  end;

  if ( Sender = lbxTags ) and
    ( lbxReferences.Visible ) then
  begin
    ShowReferenceList;
  end;
end;


initialization
  LastInitSelectWord := '';

end.
