{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....: Delphi IDE Extension Expert
 * File.......: AExpert.pas
 * Author.....: Anton A. Mints
 * Date.......: 27 June 1997
 * Description:
 *
 * Originally the code was taken from UExpert
    >  Copyright (c) 1996 Valentin Sanchez Izquierdo
    >  CopyRight (c) 1996 ValClip Group
    >  This Expert is FreeWare,use on your risk
    >  Email: valemari@ctv.es
    >  http://ourworld.compuserve.com/homepages/VSanchez
    >  MODULO: IDE Editor Utility Expert
    >
    >  DATE : 25/08/96 3:47:51
    >
    >  NOTE : Special Tanks from John Howe 100065.55@compuserve.com
    >         for mnSeparatorClick function.
 *
 * $Archive: /IDE Expert/UNITS/aexpert.pas $
 *
 * $History: aexpert.pas $
 * 
 * *****************  Version 4  *****************
 * User: Anton        Date: 7/29/97    Time: 12:46p
 * Updated in $/IDE Expert/UNITS
 * Adupt the CreateMenu procedure for the Delphi 3.0 version. Add
 * conditional compiler directives.
 * 
 * *****************  Version 3  *****************
 * User: Anton        Date: 7/02/97    Time: 11:31a
 * Updated in $/IDE Expert/UNITS
 * Correct file header template
 * 
 * *****************  Version 2  *****************
 * User: Anton        Date: 6/27/97    Time: 3:38p
 * Updated in $/IDE Expert/UNITS
 * Remove Sourcesafe constructions from the unit's body
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/units
 *
 *************}

{$UNDEF DEBUG}

{$IFNDEF VER80}
  {$IFNDEF VER90}
    {$DEFINE D3} { Delphi 3.0 or higher }
  {$ENDIF}
{$ENDIF}


unit AExpert;

interface

uses
  // Classes
  TmplList, ExpReg,

  // System & VCL Modules
  Windows, Forms, Controls, Menus, Dialogs, Classes, ToolIntf, EditIntf,
  ExptIntf, VirtIntf, SysUtils, Messages, Graphics, StdCtrls, DB, DBTables;

type

{ TEditHeaderExpert }
  TEditHeaderExpert = class (TIExpert)
    mnInsHeader:     TIMenuItemIntf;
    mnEditSep1:      TIMenuItemIntf;
    mnSeparator:     TIMenuItemIntf;
    mnKeyExtender:   TIMenuItemIntf;
    mnTagList:       TIMenuItemIntf;
    mnEditSep2:      TIMenuItemIntf;
    mnSetShortCut:   TIMenuItemIntf;

  private
    ExpRegistry: TExpRegistry;
    TmplList: TTemplateList;
    TmplDirectory: String;

    procedure mnInsertSomthingClick (Sender: TIMenuItemIntf);
    procedure mnKeyExtenderClick (Sender: TIMenuItemIntf);
    procedure mnExpConfigureClick (Sender: TIMenuItemIntf);
    procedure mnTagListClick (Sender: TIMenuItemIntf);

    procedure CreateMenu;
    procedure FillInMemoryList (TmplList: TTemplateList);
    function EnsurePasUnit: Boolean;

  public
    constructor Create; virtual;
    destructor Destroy; override;

    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;

    procedure Execute; override;
  end;

//
// Expert Initialization function
//
function InitExpert (ToolServices: TIToolServices;
  Registerproc: TExpertRegisterproc;
  var Terminate: TExpertTerminateProc): Boolean; export; stdcall;

//
// Expert Registery function
//
procedure Register;

implementation

uses
  // Classes
  TagLst,

  // Forms Modules
  TagBrws, ExpSet, ExpSplsh,

  // Units
  EdReader, TagPrsr, GetSSVer;

const
  CRLF = #13#10;

{*******************************************************************}
{*                    Class  TEditHeaderExpert                     *}
{*******************************************************************}

constructor TEditHeaderExpert.Create;
begin
  inherited Create;

  ExpRegistry := TExpRegistry.Create;

  // Create menu structure under IDE "Edit" menu
  CreateMenu;

  // All templates has to be in Delphi StartUp directory
  TmplDirectory := ExtractFileDir (Application.ExeName);

  // Create empty tempate list
  TmplList := TTemplateList.Create;

  // Read the tag list from the database into the memory
  FillInMemoryList (TmplList);
end;

destructor TEditHeaderExpert.Destroy;
begin
  TmplList.Free;
  ExpRegistry.Free;
  mnInsHeader.Free;
  mnEditSep1.Free;
  mnSeparator.Free;
  mnTagList.Free;
  mnKeyExtender.Free;
  mnEditSep2.Free;
  mnSetShortCut.Free;
  inherited Destroy;
end;


{*************
* Name.......: CreateMenu
* Description: Create menu structure under Edit menu
*}
procedure TEditHeaderExpert.CreateMenu;
var
  mnMain:     TIMainMenuIntf;
  mnEdit:     TIMenuItemIntf;
  mnEditPrnt: TIMenuItemIntf;

begin
  mnMain := ToolServices.GetMainMenu;

  if Assigned (mnMain) then
  try
    // Find 'Edit' parent menu
  {$IFDEF D3}
    mnEdit := mnMain.FindMenuItem ('EditMenu');
  {$ELSE}
    mnEdit := mnMain.FindMenuItem ('EditObjectItem');
  {$ENDIF}

    if Assigned (mnEdit) then
    try
    {$IFDEF D3}
      mnEditSep1 := mnEdit.InsertItem(mnEdit.GetIndex+1, '-', 'EditExpSepa1', '', 0, 0, 0, [mfVisible], NIL);
      mnInsHeader := mnEdit.InsertItem(mnEditSep1.GetIndex+1, 'I&nsert File Header',
                              'InsHeaderItem', '', ExpRegistry.InsHeaderSCut, 0, 0, [mfEnabled, mfVisible], mnInsertSomthingClick);
      mnSeparator:= mnEdit.InsertItem(mnInsHeader.GetIndex+1, 'Insert &Separator',
                              'InsSeparatorItem','', ExpRegistry.InsSeparSCut, 0, 0, [mfEnabled, mfVisible], mnInsertSomthingClick);
      mnTagList :=  mnEdit.InsertItem(mnSeparator.GetIndex+1, 'Browse &Tags',
                              'TagListItem','', ExpRegistry.BrowseTagSCut, 0, 0, [mfEnabled, mfVisible], mnTagListClick);
      mnKeyExtender := mnEdit.InsertItem(mnTagList.GetIndex+1, '&Apply template',
                              'EditKeyExtenderItem','', ExpRegistry.ApplyTemplSCut, 0, 0, [mfEnabled, mfVisible], mnKeyExtenderClick);
      mnEditSep2 := mnEdit.InsertItem(mnKeyExtender.GetIndex+1, '-', 'EditExpSepa2', '', 0, 0, 0, [mfVisible], NIL);
      mnSetShortCut := mnEdit.InsertItem(mnEditSep2.GetIndex+1, 'CEO Expert Configure...',
                              'EditShortCut', '', 0, 0, 0, [mfEnabled, mfVisible], mnExpConfigureClick);      // mnEditPrnt := mnEdit.GetParent;
      mnEditPrnt := nil;
    {$ELSE}
      mnEditPrnt := mnEdit.GetParent;
    {$ENDIF}

      if Assigned (mnEditPrnt) then
      try
        mnEditSep1 := mnEditPrnt.InsertItem(mnEdit.GetIndex+1, '-', 'EditExpSepa1', '', 0, 0, 0, [mfVisible], NIL);
        mnInsHeader := mnEditPrnt.InsertItem(mnEditSep1.GetIndex+1, 'I&nsert File Header',
                                'InsHeaderItem', '', ExpRegistry.InsHeaderSCut, 0, 0, [mfEnabled, mfVisible], mnInsertSomthingClick);
        mnSeparator:= mnEditPrnt.InsertItem(mnInsHeader.GetIndex+1, 'Insert &Separator',
                                'InsSeparatorItem','', ExpRegistry.InsSeparSCut, 0, 0, [mfEnabled, mfVisible], mnInsertSomthingClick);
        mnTagList :=  mnEditPrnt.InsertItem(mnSeparator.GetIndex+1, 'Browse &Tags',
                                'TagListItem','', ExpRegistry.BrowseTagSCut, 0, 0, [mfEnabled, mfVisible], mnTagListClick);
        mnKeyExtender := mnEditPrnt.InsertItem(mnTagList.GetIndex+1, '&Apply template',
                                'EditKeyExtenderItem','', ExpRegistry.ApplyTemplSCut, 0, 0, [mfEnabled, mfVisible], mnKeyExtenderClick);
        mnEditSep2 := mnEditPrnt.InsertItem(mnKeyExtender.GetIndex+1, '-', 'EditExpSepa2', '', 0, 0, 0, [mfVisible], NIL);
        mnSetShortCut := mnEditPrnt.InsertItem(mnEditSep2.GetIndex+1, 'CEO Expert Configure...',
                                'EditShortCut', '', 0, 0, 0, [mfEnabled, mfVisible], mnExpConfigureClick);
      finally
        mnEditPrnt.Free;
      end;
    finally
      mnEdit.Free;
    end;
  finally
    mnMain.Free;
  end;

end;


{*************
* Name.......: FillInMemoryList
* Description: Read the tags from the database into the memory
*}
procedure TEditHeaderExpert.FillInMemoryList (TmplList: TTemplateList);
var
  TmplItemRec: TTemplateItemRec;
  DBContainer: TTable;

begin
  // Cleare the list first
  TmplList.Clear;

  // Fill list with templates stored in database
  try
    DBContainer := TTable.Create (Application);
    DBContainer.DatabaseName := TmplDirectory;
    DBContainer.TableName := 'Key_Exp.db';

    //---
    try
      DBContainer.Open;
      while not DBContainer.EOF do
      begin
        TmplItemRec.TmplKey := DBContainer.FieldByName('KEY STRING').AsString;
        TmplItemRec.TmplTxt := DBContainer.FieldByName('TEMPLATE').AsString;
        TmplList.Add (TmplItemRec);

        DBContainer.Next;
      end;
    except
      on E:Exception do
        ShowMessage ('KEY_EXP.DLL: ' + E.Message);
    end;

  finally
    DBContainer.Close;
    DBContainer.Free;
  end;
end;


{*************
* Name.......: EnsurePasUnit
* Description: Check the Pascal unit has a focus.
*}
function TEditHeaderExpert.EnsurePasUnit: Boolean;
var
  FileName: String;
begin
  // Assume a pascal unit by default
  Result := True;

  // Get the file name
  FileName := Toolservices.GetCurrentFile;

  if ( UpperCase(ExtractFileExt (FileName)) <> '.PAS' ) then
  begin
    // Tell em not to be stupid
    ShowMessage('Pascal source must have focus');
    Result := False;
  end;

  // If file is not open, try to open it
  if not Toolservices.IsFileOpen(FileName) and
    not Toolservices.OpenFile(FileName) then
  begin
    // Tell em we failed
    MessageDlg(Format('Can''t open %s',[FileName]), mtError,[mbOk],0);
    Result := False;
  end;
end;


{*************
* Name.......: mnInsertSomthing (OnClick EVENT)
* Description: Insert custom file separator.
*}
procedure TEditHeaderExpert.mnInsertSomthingClick (Sender: TIMenuItemIntf);
var
  MoInt : TIModuleInterface;
  EdInt : TIEditorInterface;
  EdWrit: TIEditWriter;
  EdRead: TIEditReader;
  EdPos : TEditPos;
  Position: Longint;

  procedure Add (Line: String);
  begin
    EdWrit.Insert(PChar (Line + CRLF));
  end;

begin
  // No toolservices, no can do
  if Toolservices = nil then Exit;

  // No PAS unit
  if not EnsurePasUnit then Exit;

  // Get the module interface for the file
  with Toolservices do
    MoInt := GetModuleInterface (GetCurrentFile);
  try
    EdInt := MoInt.GetEditorInterface;

    // Get the current cursor position
    if ( Sender.GetName <> 'InsHeaderItem' ) then
      begin
        EdPos := IEIGetViewPos (EdInt);
      end
    // or on the top of the file for the file header
    else
      begin
        EdPos.Line := 1;
        EdPos.Col := 1;
      end;


    EdRead := EdInt.CreateReader;
    try
      // Jump to the required line
      if not GotoLine(EdRead, EdPos, Position) then Exit;
    finally
      EdRead.Free;
    end;


    EdWrit := EdInt.CreateWriter;
    try
      EdWrit.CopyTo (Position);

      if ( Sender.GetName = 'InsSeparatorItem' ) then
        begin
          Add ('//-----------------------------------------'+
            '-----------------------------//');
        end
      else
        begin
          //
          // CEO File Header
          //
          Add ('{*************  ');
          Add (' *              ');
          Add (' * Copyright (c) 1997 CEO Systems Inc.');
          Add (' *              ');
          Add (' * Project....: ');
          Add (' * File.......: ' + ExtractFileName (ToolServices.GetCurrentFile));
          Add (' * Author.....: ' + ExpRegistry.Author);
          Add (' * Date.......: ' + FormatDateTime ('dd mmmm yyyy', Date));
          Add (' * Description: ');
          Add (' *              ');
          Add (' * $' + 'Archive: $ ');
          Add (' *              ');
          Add (' * $' + 'History: $ ');
          Add (' *              ');
          Add (' *************} ');
        end;
    finally
      EdWrit.Free;
    end;

  finally
    MoInt.Free;
    EdInt.Free;
  end;
end;


{*************
* Name.......: mnTagList (OnClick EVENT)
* Description: Display Tag List
*}
procedure TEditHeaderExpert.mnTagListClick (Sender: TIMenuItemIntf);
var
  ModuleInterface: TiModuleInterface; // module interface
  EditorInterface: TiEditorInterface; // editor interface
  EditReader: TIEditReader;
  TopPos, CursorPos: TEditPos;
  NewLineNum: Integer;
  FileName: String;
  SelWord: String;
  lstTags, lstImplPos: TTagList;

begin
  // No toolservices, no can do.
  if ( Toolservices = nil ) then Exit;

  // No PAS unit
  if not EnsurePasUnit then Exit;

  // Get current file name.
  FileName := Toolservices.GetCurrentFile;

  // Get the module interface for the file
  ModuleInterface := Toolservices.GetModuleInterface(FileName);
  try
    // get the editor interface
    EditorInterface := ModuleInterface.GetEditorInterface;
    try
      lstTags := TTagList.Create;
      lstImplPos := TTagList.Create;
      EditReader := EditorInterface.CreateReader;

      try
        TopPos := IEIGetTopPos (EditorInterface);
        CursorPos := IEIGetViewPos (EditorInterface);

        // Get the worg under cursor
        SelWord := ReadCentralWord (EditReader, CursorPos);

        {$IFDEF DEBUG}
          ShowMessage ('Scan ' + FileName + ' for tags...');
        {$ENDIF}

        FillTagList (FileName, lstTags, lstImplPos);

        with TFrmTagBrowser.Create (Application) do
        try
          NewLineNum := ShowModal (EditorInterface,
            FileName, lstTags, lstImplPos, SelWord);

          if ( NewLineNum > 0 ) then
          begin
            CursorPos.Line := NewLineNum;
            CursorPos.Col := 1;

            TopPos := CursorPos;
          end;

          // Scroll view to the selected line
          IEISetTopPos (EditorInterface, TopPos);

          // Move cursor on set position
          IEISetCursorPos (EditorInterface, CursorPos);
        finally
          Free;
        end;

      finally
        lstTags.Free;
        lstImplPos.Free;
        EditReader.Free;
      end;
    finally
      EditorInterface.Free;
    end;
  finally
    ModuleInterface.Free;
  end;
end;


{*************
* Name.......: mnKeyExtender (OnClick EVENT)
* Description: Apply template
*}
procedure TEditHeaderExpert.mnKeyExtenderClick (Sender: TIMenuItemIntf);
var
  ModuleInterface: TiModuleInterface; // module interface
  EditorInterface: TiEditorInterface; // editor interface
  EditReader: TIEditReader;
  EditWriter: TiEditWriter;           // Edit Writer
  EditPos:    TEditPos;               // the editorposition
  KeyString:  String;
  FileName:   String;
  FirstPos:   LongInt;
  EditorLine: String;

  {--================--}

  KeyExpandText: String;
  KeyExpandLine: String;
  KeyExpandPos: Integer;
  I, KeyOffset: Integer;
  CursorToEnd: Boolean;
  TmplIndex: Integer;

begin
  // No toolservices, no can do.
  if ( Toolservices = nil ) then Exit;

  // No PAS unit
  if not EnsurePasUnit then Exit;

  // Get current file name.
  FileName := Toolservices.GetCurrentFile;

  // Get the module interface for the file
  ModuleInterface := Toolservices.GetModuleInterface(FileName);
  try
    // get the editor interface
    EditorInterface := ModuleInterface.GetEditorInterface;
    try
      EditPos := IEIGetViewPos (EditorInterface);

      EditReader := EditorInterface.CreateReader;
      try
        if not GotoLine (EditReader, EditPos, FirstPos) then Exit;
        KeyString := GetLeftWord (EditReader, EditPos, FirstPos);
        {$IFDEF DEBUG}
          ShowMessage (KeyString);
        {$ENDIF}
      finally
        EditReader.Free;
      end;

      TmplIndex := TmplList.IndexOf (KeyString);

      if ( TmplIndex >= 0 ) then
      try
        { If we found key string expand template text. }
        if ( TmplIndex >= 0 ) then
        begin
          // Count offset of the keyword
          {$IFDEF DEBUG}
            ShowMessage ('Offces = ' + IntToStr (EditPos.Col));
          {$ENDIF}
          KeyOffset := EditPos.Col;

          EditWriter := EditorInterface.CreateWriter;
          try
            with EditWriter do
            begin
              // Move down to the first char of the line
              CopyTo (FirstPos + EditPos.Col - 1);
              DeleteTo (FirstPos + EditPos.Col - 1 + Length (KeyString));

              // Get stored template
              KeyExpandText := TmplList[TmplIndex].TmplTxt;
              KeyExpandPos := 1;
              CursorToEnd := True;

              if (KeyExpandText) = 'VERSION' then
                KeyExpandText := 'V' + IntToStr (GetSSVersionNum (FileName)) +
                  ExpRegistry.AuthorTAG + ' ';

              KeyExpandLine := '';
              repeat
                if ( KeyExpandText [KeyExpandPos] = '~' ) then
                  begin
                    EditPos.Col := EditPos.Col + Length (KeyExpandLine);
                    CursorToEnd := False;
                  end

                else if ( KeyExpandText [KeyExpandPos] = #13 ) then
                  begin
                    // Goto next line
                    if CursorToEnd then
                      begin
                        EditPos.Line := EditPos.Line + 1;
                        EditPos.Col  := 1;
                      end;
                    // Insert line of text
                    Insert (PChar(KeyExpandLine + CRLF));

                    // Add offset before each line
                    KeyExpandLine := '';
                    for I := 1 to KeyOffset-1 do
                      KeyExpandLine := KeyExpandLine + ' ';

                    Inc (KeyExpandPos);
                  end

              	else if( KeyExpandPos = Length (KeyExpandText)) then
                  begin
                    // Insert line of text + last character
                    Insert (PChar(KeyExpandLine + KeyExpandText [KeyExpandPos]));
                    if CursorToEnd then
                      EditPos.Col := EditPos.Col + Length (KeyExpandLine)
                  end

                else
                  KeyExpandLine := KeyExpandLine + KeyExpandText [KeyExpandPos];

                Inc (KeyExpandPos);

              until ( KeyExpandPos > Length (KeyExpandText) );

            end;

            // Move cursor on set position
            IEISetCursorPos (EditorInterface, EditPos);

          finally
            EditWriter.Free;
          end;
        end;

      finally
      end;

    finally
      EditorInterface.Free;
    end;

  finally
    ModuleInterface.Free;
  end;
end;


{*************
* Name.......: mnExpConfigure (OnClick EVENT)
*}
procedure TEditHeaderExpert.mnExpConfigureClick (Sender: TIMenuItemIntf);
begin
  with TFrmExpertSettings.Create (Application) do
  try
    if ( ShowModal (TmplDirectory) = mrOk ) then
    begin
      // Reset menu short cuts
      mnInsHeader.SetShortCut   (ExpRegistry.InsHeaderSCut);
      mnSeparator.SetShortCut   (ExpRegistry.InsSeparSCut);
      mnTagList.SetShortCut     (ExpRegistry.BrowseTagSCut);
      mnKeyExtender.SetShortCut (ExpRegistry.ApplyTemplSCut);
      // Reread the InMemory list
      FillInMemoryList (TmplList);
    end;
  finally
    Free;
  end;
end;


{*************
* Name.......: Execute
*}
procedure TEditHeaderExpert.Execute;
begin
  {nada}
end;


{*************
* Name.......: GetName
*}
function TEditHeaderExpert.GetName: String;
begin
  Result:='CEO Editor Expert';
end;


{*************
* Name.......: GetStyle
*}
function TEditHeaderExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;


{*************
* Name.......: GetState
*}
function TEditHeaderExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;


{*************
* Name.......: GetIDString
*}
function TEditHeaderExpert.GetIDString: String;
begin
  Result:='CEO_Systems.Key_Expert';
end;


{*************
* Name.......: Register
*}
procedure Register;
begin
  RegisterLibraryExpert (TEditHeaderExpert.Create);
end;


procedure TerminateProc;
begin
  // Nothing to do on terminate
end;


{*************
* Name.......: InitExpert
* Description: Expert Initialization
*}
function InitExpert (ToolServices: TIToolServices;
  Registerproc: TExpertRegisterproc;
  var Terminate: TExpertTerminateProc): Boolean; export; stdcall;

var
  EditHeaderExpert: TEditHeaderExpert;

begin
  Result := ( ExptIntf.ToolServices = nil );

  if not Result then Exit;

  ExptIntf.ToolServices:=ToolServices;

  if Assigned(ToolServices) then
  begin
    Application.Handle := ToolServices.GetParentHandle;

    // Show the splash form for 1 sec
    with TFrmExpertSplash.Create (Application) do
    begin
      Show;
      Update;
    end;
  end;

  // Set terminate procedure for future purposes
  Terminate := TerminateProc;

  // Create ...
  EditHeaderExpert := TEditHeaderExpert.Create;

  // and register Expert
  RegisterProc (EditHeaderExpert);
end;

end.
