{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....:
 * File.......: EdReader.pas
 * Autor......: Anton A. Mitns
 * Date.......: 27 May 1997
 * Description: A couple of funcitons to easy work with TIEditReader class
 *
 * $Archive: /ide expert/units/edreader.pas $
 *
 * $History: edreader.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/units
 *
 *************}

unit EdReader;

interface

uses
  EditIntf, Classes;

function IEIGetViewPos (EditorInterface: TIEditorInterface): TEditPos;
function IEIGetTopPos (EditorInterface: TIEditorInterface): TEditPos;
procedure IEISetCursorPos (EditorInterface: TIEditorInterface;
  EditPos: TEditPos);
procedure IEISetTopPos (EditorInterface: TIEditorInterface;
  EditPos: TEditPos);

function IEIReadLeftWord (EditorInterface: TIEditorInterface;
  EditPos: TEditPos): String;

function GotoLine (EditReader: TIEditReader;
  EditPos: TEditPos; var Position: LongInt): Boolean;

function ReadLine (EditReader: TIEditReader;
  EditPos: TEditPos): String;

function GetLeftWord (EditReader: TIEditReader;
  var EditPos: TEditPos; var FirstPos: LongInt): String;

function GetCentralWord (EditReader: TIEditReader;
  var EditPos: TEditPos; var FirstPos: LongInt): String;

function ReadLeftWord (EditReader: TIEditReader;
  var EditPos: TEditPos): String;

function ReadCentralWord (EditReader: TIEditReader;
  var EditPos: TEditPos): String;

implementation

uses
  SysUtils, Dialogs;

function IEIGetViewPos (EditorInterface: TIEditorInterface): TEditPos;
var
  EditView: TIEditView;
begin
  EditView := EditorInterface.GetView(0);
  try
    // get the current cursor position
    Result := EditView.GetPos(0);
  finally
    EditView.Free;
  end;
end;


function IEIGetTopPos (EditorInterface: TIEditorInterface): TEditPos;
var
  EditView: TIEditView;
begin
  EditView := EditorInterface.GetView(0);
  try
    // get the current cursor position
    Result := EditView.GetPos(1);
  finally
    EditView.Free;
  end;
end;


procedure IEISetCursorPos (EditorInterface: TIEditorInterface;
  EditPos: TEditPos);
var
  EditView: TIEditView;
begin
  EditView := EditorInterface.GetView(0);
  try
    EditView.CursorPos := EditPos;
  finally
    EditView.Free;
  end;
end;


procedure IEISetTopPos (EditorInterface: TIEditorInterface;
  EditPos: TEditPos);
var
  EditView: TIEditView;
begin
  EditView := EditorInterface.GetView(0);
  try
    EditView.TopPos := EditPos;
  finally
    EditView.Free;
  end;
end;


function IEIReadLeftWord (EditorInterface: TIEditorInterface;
  EditPos: TEditPos): String;
var
  EditReader: TIEditReader;
begin
  EditReader := EditorInterface.CreateReader;
  try
    Result := ReadLeftWord (EditReader, EditPos);
  finally
    EditReader.Free;
  end;
end;


// Description: Goto line from file start
function GotoLine (EditReader: TIEditReader;
  EditPos: TEditPos; var Position: LongInt): Boolean;
var
  Eof: Boolean;
  LF: array[0..1] of Char;

begin
  // Assume failure
  Result := False;
  EOF := False;

  // Zap char pos and buffer
  Position := 0;
  LF[0] := #00;

  // If there is a first line we are there already
  if ( EditPos.Line = 1 ) then
  begin
    Result := True;
    Exit;
  end;

  // While not done
  while not ( EOF ) do
  begin
    while not ( EOF ) and ( LF[0] <> #10 ) do
    try
      // read a character
      EOF := ( EditReader.GetText (Position, LF, 1) <> 1 );
      // bump Editor position
      Inc (Position);
    except
      // assume EOF on exception
      EOF := True;
    end;

    Dec (EditPos.Line);

    // If EOL then bump count
    Inc (Position);
    LF[0] := #0;

    // If this is the line we want leave loop
    if ( EditPos.Line = 1 ) then
    begin
      EOF := True;
      Result := True;
      Dec (Position);
    end;

  end;

end;


// Description: Read a return a line from the Editor

function ReadLine (EditReader: TIEditReader; EditPos: TEditPos): String;
var
  FirstPos: LongInt;
  CharPos: Integer; // Current Position in the result string
  Eof: Boolean;

begin
  // Jumt to the first pos in the line
  if not GotoLine (EditReader, EditPos, FirstPos) then
  begin
    Result := '';
    Exit;
  end;

  // Longest Line !!!!
  SetLength (Result, 1024);

  // Begin at the begining
  CharPos := 1;

  repeat
    try
      Eof := ( EditReader.GetText(FirstPos, @Result [CharPos], 1) <> 1 );
    except
      begin
        Eof := True;
        Result := '';
      end;
    end;

    // Read one so increment the buffer position
    Inc(FirstPos);

    // was it a LineFeed and if so...
    if ( Result [CharPos] = #10 ) then
    begin
      Result [CharPos] := #0;
      SetLength (Result, CharPos - 1);
      Exit;
    end;

    { Added new char into new position. }
    Inc(CharPos);

  until Eof or (CharPos > 1024);

  // If we get here we are Eof so return ''.
  Result := '';

end;


function GetLeftWord (EditReader: TIEditReader;
  var EditPos: TEditPos; var FirstPos: LongInt): String;
var
  SpacePos: Word;
  EOLPos: Word;
  LeftLendth: Word;
  LeftWord: String;

begin
  // Longest Line !!!!
  SetLength (LeftWord, 1024);

  // Looking for the first non empty character.
  LeftLendth := EditReader.GetText(FirstPos, @LeftWord[1], EditPos.Col);
  EOLPos := Pos (Chr(10), LeftWord);

  if ( EditPos.Col > EOLPos ) and
      ( EOLPos > 0 ) then
    LeftLendth := EOLPos-1;

  LeftWord [LeftLendth+1] := #0;
  SetLength (LeftWord, LeftLendth);
  LeftWord := TrimRight(LeftWord);

  // Look for next left space.
  SpacePos := Length (LeftWord);
  repeat
    Dec (SpacePos);
  until ( SpacePos = 0 ) or ( LeftWord[SpacePos] = #32 );

  // Set Edit column pos as position of the begining of the word
  EditPos.Col := SpacePos + 1;

  Result := Copy (LeftWord, EditPos.Col, LeftLendth);
end;


{*************
* Name.......: GetCentralWord
*}
function GetCentralWord (EditReader: TIEditReader;
  var EditPos: TEditPos; var FirstPos: LongInt): String;
var
  SpacePos: Word;
  EOLPos: Word;
  LineLen: Word;
  CentralWord: String;

begin
  // Longest Line !!!!
  SetLength (CentralWord, 1024);

  // Looking for the first non empty character.
  LineLen := EditReader.GetText(FirstPos, @CentralWord [1], 1024);
  EOLPos := Pos (Chr(10), CentralWord);

  if ( EOLPos > 0 ) then
    LineLen := EOLPos-1;

  CentralWord [LineLen+1] := #0;
  SetLength (CentralWord, LineLen);

  // Look for next left token.
  if ( LineLen < EditPos.Col ) then
    SpacePos := LineLen
  else
    SpacePos := EditPos.Col;

  repeat
    Dec (SpacePos);
  until ( SpacePos = 0 ) or
    ( CentralWord[SpacePos] in [' ','.',';','[',']','(',')',',',':']);

  // Set Edit column pos as position of the begining of the word
  EditPos.Col := SpacePos + 1;

  // Cut left part
  Result := Copy (CentralWord, EditPos.Col, LineLen);

  // Looking for rigth token pos
  SpacePos := 0;
  repeat
    Inc (SpacePos);
  until ( SpacePos = Length (Result) ) or
    ( Result[SpacePos] in [' ','.',';','[',']','(',')',',',':']);

  Result := Copy (Result, 1, SpacePos-1);
end;


function ReadLeftWord (EditReader: TIEditReader;
  var EditPos: TEditPos): String;
var
  FirstPos: LongInt;
begin
  // Jumt to the first pos in the line
  if not GotoLine (EditReader, EditPos, FirstPos) then
    Result := ''
  else
    Result  := GetLeftWord (EditReader, EditPos, FirstPos);
end;


{*************
* Name.......: ReadCentralWord
*}
function ReadCentralWord (EditReader: TIEditReader;
  var EditPos: TEditPos): String;
var
  FirstPos: LongInt;
begin
  // Jumt to the first pos in the line
  if not GotoLine (EditReader, EditPos, FirstPos) then
    Result := ''
  else
    Result := GetCentralWord (EditReader, EditPos, FirstPos);
end;

end.
