{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....:
 * File.......: RefPrsr.pas
 * Autor......: Anton A. Mitns
 * Date.......: 06 June 1997
 * Description: Scan pascal unit to get the list of the references
 *
 * $Archive: /ide expert/units/refprsr.pas $
 *
 * $History: refprsr.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/units
 *
 *************}

unit RefPrsr;

interface

uses
  EditIntf, Classes, TagLst;

const
  fmtTagLine = '%4d%1d%s';

procedure FindReference (FileName: String;
  Substring: String; var lstRef: TTagList;
  lstImpl: TTagList);

implementation

uses
  SysUtils, Tokens;


{*************
* Name.......: FindReference
* Description: Parse the list to find all references.
*}
procedure FindReference (FileName: String;
  Substring: String; var lstRef: TTagList;
  lstImpl: TTagList);
var
  F: Text;
  Line: String;
  LineNum: Integer;
  RefItem: RTagItem;

  function EnsureSubstring (var Line: String): Boolean;
  var
    I, MaxTokenNum: Integer;

  begin
    // Assume failure
    Result := False;

    // If there is a substring, perform detail analisis
    if Pos (SubString, Line) > 0 then
    begin
      // Remove leading spaces
      Line := TrimLeft (Line);

      // Exact match
      if ( Line = SubString ) then
      begin
        Result := True;
        Exit;  
      end;

      // Make sure it isn't declaration, but call
      if ( Pos ('*', Line) > 0 ) or
         ( Pos ('procedure', Line) > 0 ) or
         ( Pos ('function', Line) > 0 ) or
         ( Pos ('constructor', Line) > 0 ) or
         ( Pos ('destructor', Line) > 0 ) or
         ( Pos ('property', Line) > 0 ) then
        Exit;

      // Substring must be a whole word
      MaxTokenNum := NumToken (Line);
      for I := 1 to MaxTokenNum do
        if ( Token (Line, I) = Substring ) then
        begin
          Result := True;
          Exit;
        end;
    end;
  end;

begin
  LineNum := 0;

  AssignFile (F, FileName);
  Reset (F);

  repeat
    try
      ReadLn (F, Line);
    except
      Break;
    end;

    Inc (LineNum);

    // If there is a reference found
    if EnsureSubstring (Line) then
    begin
      RefItem.tagName := Format ('%4d %s', [LineNum, Line]);
      RefItem.tagType := usNone;
      RefItem.tagSubType := usNone;
      RefItem.GroupNum := 0;
      RefItem.LineNum := LineNum;

      lstRef.Add (RefItem);
    end;

  until ( EOF (F) );

  CloseFile (F);
end;


end.
