{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....: Delphi IDE Extension Expert
 * File.......: TagPrsr.pas
 * Autor......: Anton A. Mitns
 * Date.......: 06 June 1997
 * Description: Scan pascal unit to full fill tag list.
 *
 * $Archive: /ide expert/units/tagprsr.pas $
 *
 * $History: tagprsr.pas $
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 6/27/97    Time: 1:37p
 * Created in $/ide expert/units
 *
 *************}

unit TagPrsr;

interface

uses
  EditIntf, Classes, TagLst;

const
  fmtTagLine = '%4d%1d%s';

procedure FillTagList (FileName: String;
  var lstDecl, lstImpl: TTagList);

function FindInTagList (lstTag: TTagList; Word: String;
  StartIndex: Integer): Integer;

implementation

uses
  SysUtils, Tokens, Dialogs;

{*************
* Name.......: FillTagList
* Description: Parse the list to find al tags.
*}
procedure FillTagList (FileName: String;
  var lstDecl, lstImpl: TTagList);
var
  F: Text;
  Tag: String;
  Line: String;
  LineNum: Integer;
  TagItem: RTagItem;
  LookingForEnd: Boolean;
  classSection: TTagType;


  {*************
  * Name.......: GetFirstToken
  *}
  function GetFirstToken (Line: String): String;
  var
    CharPos: Integer;
  begin
    //    
    CharPos := Pos (' ', Line);

    if ( CharPos > 0 ) then
      Result := Copy (Line, 1, CharPos-1)
    else
      Result := Line;

    //     ;
    CharPos := Pos (';', Result);

    if ( CharPos > 0 ) then
      Result := Copy (Result, 1, CharPos-1);
  end;


  {*************
  * Name.......: IsSectionStarted
  *}
  function IsSectionStarted (Tag, Line: String): Boolean;
  begin
    Result := ( Tag = 'procedure' ) or
      ( Tag = 'function' ) or
      ( Tag = 'property' ) or
      ( Tag = 'constructor' ) or
      ( Tag = 'destructor' ) or
      (
        ( Pos ('class', Line) > 0 ) and
        ( Pos ('=', Line) > 0 ) and
        ( Pos ('(', Line) > 0 ) and
        ( Pos (')', Line) > 0 )
      );
  end;


  {*************
  * Name.......: IsSectionFinished
  *}
  function IsSectionFinished (Tag, Line: String): Boolean;
  begin
    Result := ( Tag = 'end' ) or
      ( Tag = 'begin' ) or
      ( Tag = 'private' ) or
      ( Tag = 'public' ) or
      ( Tag = 'protected' ) or
      ( Tag = 'published' ) or
      ( Tag = 'private' ) or
      (( Tag = 'var' ) and (Trim (Line) = 'var')) or
      ( Pos ('record', Line) > 0 );
  end;


begin
  // Initialize variables
  classSection := usNone;
  LookingForEnd := False;
  LineNum := 0;
  Line := '';

  // Initialize structure
  TagItem.tagName := '';
  TagItem.tagType := usNone;
  TagItem.tagSubType := usNone;
  TagItem.GroupNum := 0;
  TagItem.LineNum := 0;

  AssignFile (F, FileName);
  Reset (F);

  repeat
    try
      ReadLn (F, Line);
    except
      Break;
    end;

    Inc (LineNum);

    // Remove leading spaces
    Line := TrimLeft (Line);

    if ( Length (Line) >= 3 ) then
    begin
      // Get line tag.
      Tag := GetFirstToken (Line);

      //     
      if IsSectionStarted (Tag, Line) then
        begin
          //     
          if ( TagItem.tagName <> '' ) then
          begin
            //   ,    tagSubType  
            if ( TagItem.tagType = usClass ) then
              TagItem.tagSubType := usNone;

            //   .  ,    ,
            //  TAG,  .
            if LookingForEnd then
              lstDecl.Add (TagItem);
            //else
            // lstImpl.Add (TagItem);
          end;

          //   
          TagItem.tagSubType := classSection;
          TagItem.LineNum := LineNum;

          // Determine the TAG type
          if ( Tag = 'procedure' ) then
            TagItem.tagType := usProcedure
          else if ( Tag = 'function' ) then
            TagItem.tagType := usFunction
          else if ( Tag = 'property' ) then
            TagItem.tagType := usProperty
          else if ( Tag = 'constructor' ) then
            TagItem.tagType := usConstructor
          else if ( Tag = 'destructor' ) then
            TagItem.tagType := usDestructor
          else
            begin
              TagItem.tagType := usClass;
              //     -  
              classSection := usEvents;
            end;

          // Remove TAG from the name
          if ( TagItem.tagType <> usClass ) then
            begin
              TagItem.tagName := TrimLeft (Copy (Line, Length (Tag)+1, 1024));

              // Remove class name form class method declaration
              if Pos ('.', TagItem.tagName) > 0 then
                TagItem.tagName := Copy (TagItem.tagName,
                  Pos ('.', TagItem.tagName)+1, 1024);
            end
          else
            TagItem.tagName := Line;

          //   ,    
          LookingForEnd := True;
        end

      //    ,    
      else if ( LookingForEnd ) then
      begin
        //    
        if IsSectionFinished (Tag, Line) then
          begin
            //   ,    tagSubType  
            if ( TagItem.tagType = usClass ) then
              TagItem.tagSubType := usNone;

            //   .  ,
            //   VAR  BEGIN,
            //   .
            if ( Tag = 'begin' ) or ( Tag = 'var' ) then
              lstImpl.Add (TagItem)
            else
              lstDecl.Add (TagItem);

            //     
            //  .
            if ( Tag <> 'private' ) and
                ( Tag <> 'public' ) and
                ( Tag <> 'protected' ) and
                ( Tag <> 'published' ) then
              classSection := usNone;

            //   ,       
            LookingForEnd := False;
          end
        else
          //  TAG name
          TagItem.tagName := TagItem.tagName + ' ' + Line;
      end;

      //    
      if ( Tag = 'private' ) then
        classSection := usPrivate
      else if ( Tag = 'public' ) then
        classSection := usPublic
      else if ( Tag = 'protected' ) then
        classSection := usProtected
      else if ( Tag = 'published' ) then
        classSection := usPublished
      else if ( Tag = 'private' ) then
        classSection := usPrivate;

    end;

  until ( EOF(F) );

  CloseFile (F);
end;


{*************
* Name.......: FindInTagList
*}
function FindInTagList (lstTag: TTagList; Word: String;
  StartIndex: Integer): Integer;
var
  Index: Integer;
  WordLen: Integer;

begin
  // Assume fail
  Result := -1;

  // Length of the word
  WordLen := Length (Word);

  // Chech Start index is in valid.
  if (StartIndex < 0) or
      (StartIndex > lstTag.Count-1) then
    StartIndex := 0;

  // Lower case it
  Word := LowerCase (Word);

  // Nothing to check
  if ( lstTag.Count = 0 ) then Exit;

  for Index := StartIndex to lstTag.Count-1 do
    if Word = LowerCase (Copy (lstTag[Index].tagName, 1, WordLen)) then
    begin
      Result := Index;
      Break;
    end;
end;

end.
