{*************
 *
 * Copyright 1999 Anton A. Mints
 *
 * Project....: Freeware components
 * File.......: StrinGr.pas
 * Author.....: Anton A. Mints
 * Date.......: 19 January 1999
 * Description: String grid with save and restore content
 *
 * $Archive: $
 *
 * $History: $
 * 
 *
 *************}

unit StringGr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids;

type
  AStringGrid = class(TStringGrid)
  private
    FLoadData: Boolean;

  protected
    procedure ReadDataCells(Reader: TReader);
    procedure WriteDataCells(Writer: TWriter);
    procedure DefineProperties(Filer: TFiler); override;

  published
    property LoadData: Boolean read FLoadData write FLoadData;
    
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [AStringGrid]);
end;

{**************************************************************************
*                        CLASS AStringGrid                                *
**************************************************************************}

procedure AStringGrid.ReadDataCells(Reader: TReader);
var
  iCol, iRow: Integer;
  TmpStr: String;

begin
  Reader.ReadListBegin;
  while not Reader.EndOfList do
  begin
    TmpStr := Reader.ReadString;

    if (not (csDesigning in ComponentState) and
      not FLoadData) then Continue;

    iCol := StrToIntDef(Copy(TmpStr, 1, 6), -1);
    iRow := StrToIntDef(Copy(TmpStr, 7, 6), -1);

    if ((iCol >= 0) and (iRow >= 0) and
        (iRow < RowCount) and (iCol < ColCount)) then
      Cells[iCol, iRow] := Copy(TmpStr, 13, Length(TmpStr));
  end;
  Reader.ReadListEnd;
end;

procedure AStringGrid.WriteDataCells(Writer: TWriter);
var
  iCol, iRow: Integer;

begin
  Writer.WriteListBegin;
  for iCol := 0 to ColCount - 1 do
    for iRow := 0 to RowCount - 1 do
      if (Cells[iCol, iRow] <> '') then
        Writer.WriteString(Format('%.6d%.6d%s',
          [iCol, iRow, Cells[iCol, iRow]]));
  Writer.WriteListEnd;
end;

procedure AStringGrid.DefineProperties(Filer: TFiler);
var
  iCol, iRow: Integer;
  DoReadWrite: Boolean;

begin
  inherited;
  
  DoReadWrite := True;

  if (csWriting in ComponentState) then
  begin
    // Write only if grid is not empty
    DoReadWrite := False;
    for iCol := ColCount - 1 downto 0 do
      for iRow := RowCount - 1 downto 0 do
        DoReadWrite := (Cells[iCol, iRow] <> '') or (DoReadWrite)
  end;

  Filer.DefineProperty('Cells', ReadDataCells, WriteDataCells, DoReadWrite);
end;

end.
