unit funcPlotter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, Parser, Calculus;

type
  TFuncPlotter = class(TGraphicControl)
  private
    Mformula:String;
    FOnChange: TNotifyEvent;
    parser: TParser;
    FCalc: TCalculus;
    fx,fy,tx,ty:double;
    fStep:double;
    varX:TVarDef;
    fpen:TPen;
    procedure setFormula(value: String);
    procedure setMinX(value: double);
    procedure setMinY(value: double);
    procedure setMaxX(value: double);
    procedure setMaxY(value: double);
    procedure setStep(value: double);
    Function coordX(x:double):integer;
    Function coordY(y:double):integer;
    procedure SetPen(Value: TPen);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure zoomIn;
    procedure zoomOut;
    procedure translate(dx,dy:integer);
    procedure centerIn(x,y:integer);
  published
    property Height default 65;
    property Width default 65;
    property formula:String read Mformula write setFormula;
    property minX:double read fx write setMinX;
    property minY:double read fy write setMinY;
    property maxX:double read tx write setMaxX;
    property maxY:double read ty write setMaxY;
    property step:double read fStep write setStep;
    property pen:TPen read fpen write setPen;

    property OnChange:TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property Align;
    property Anchors;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Math', [TFuncPlotter]);
end;

procedure TFuncPlotter.SetFormula(Value: String);
var
   e:byte;
begin
  if MFormula <> Value then
  begin
    MFormula := Value;
    FCalc:=Parser.compile(MFormula,e);
    Invalidate;
    if assigned(FOnChange) then FOnChange(self);
  end;
end;

procedure TFuncPlotter.SetMinX(Value: Double);

begin
  if fx <> Value then
  begin
    fx := Value;
    Invalidate;
  end;
end;

procedure TFuncPlotter.SetMinY(Value: Double);

begin
  if fy <> Value then
  begin
    fy := Value;
    Invalidate;
  end;
end;

procedure TFuncPlotter.SetMaxX(Value: Double);

begin
  if tx <> Value then
  begin
    tx := Value;
    Invalidate;
  end;
end;

procedure TFuncPlotter.SetMaxY(Value: Double);

begin
  if ty <> Value then
  begin
    ty := Value;
    Invalidate;
  end;
end;

procedure TFuncPlotter.SetStep(Value: Double);

begin
  if fStep <> Value then
  begin
    fStep := Value;
    Invalidate;
  end;
end;

procedure TFuncPlotter.SetPen(Value: TPen);

begin
  FPen.Assign(Value);
  Invalidate;
end;

constructor TFuncPlotter.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  Width := 65;
  Height := 65;
  fX:=-1;
  fY:=-1;
  tX:=1;
  ty:=1;
  fstep:=0.002;
  Parser:=TParser.Create;
  Parser.standardFunctions;
  varX:=TVarDef.Create;
  varX.name:='x';
  Parser.addVar(varX);
  fCalc:=nil;
  fPen:=TPen.Create;
end;

destructor TFuncPlotter.Destroy;

begin
  Parser.Free;
  fpen.Free;
  inherited Destroy;
end;

procedure TFuncPlotter.zoomIn;
var
   mx,my:double;
begin
     mx:=(minX+maxX)/2;
     my:=(miny+maxY)/2;
     minx:=mX+3*(minX-mX)/2;
     maxx:=mX+3*(maxX-mX)/2;
     miny:=my+3*(miny-my)/2;
     maxy:=my+3*(maxy-my)/2;
     step:=3*step/2;
end;

procedure TFuncPlotter.zoomOut;
var
   mx,my:double;
begin
     mx:=(minX+maxX)/2;
     my:=(miny+maxY)/2;
     minx:=mX+2*(minX-mX)/3;
     maxx:=mX+2*(maxX-mX)/3;
     miny:=my+2*(miny-my)/3;
     maxy:=my+2*(maxy-my)/3;
     step:=2*step/3;
end;

procedure TFuncPlotter.translate(dx,dy:integer);
var
   ddx,ddy:double;
begin
     ddx:=dx*(maxX-minX)/width;
     ddy:=dy*(maxY-minY)/height;
     minx:=minx-ddx;
     miny:=miny+ddy;
     maxx:=maxx-ddx;
     maxy:=maxy+ddy;
end;

procedure TFuncPlotter.centerIn(x,y:integer);
var
   dx,dy:Integer;
begin
     dx:=CoordX((maxX+minX)/2)-x;
     dy:=CoordY((maxY+minY)/2)-y;
     translate(dx,dy);
end;

Function TFuncPlotter.coordX(x:double):integer;
begin
     coordX:=round((x-fx)/(tx-fx)*width);
end;

Function TFuncPlotter.coordY(y:double):integer;
begin
     coordY:=height-round((y-fy)/(ty-fy)*height);
end;


procedure TFuncPlotter.Paint;
Var
   x,y:double;
   err:Boolean;

begin
with Canvas do
begin
     Pen:=self.Pen;
     if (fy<0) and (ty>0) then  {draws y-axis}
       begin
            PenPos:=Point(0,coordY(0));
            LineTo(Width,coordY(0));
       end;

     if (fx<0) and (tx>0) then  {draws x-axis}
       begin
            PenPos:=Point(coordX(0),0);
            LineTo(coordX(0),Height);
       end;

     if fCalc=nil then exit;
     x:=fx;
      try
         varX.value:=x;
         y:=FCalc.eval;
         PenPos:=Point(coordX(x),coordY(y));
         err:=False;
      except
         err:=True;
      end;
     While x<tx do   {draws lines}
     begin
          try
             varX.value:=x;
             y:=FCalc.eval;
             if err then
                penPos:=Point(coordX(x),coordY(y))
             else
                LineTo(coordX(x),coordY(y));
             err:=False;
          except
             err:=True;
          end;
          x:=x+fStep;
     end;
end; {with canvas}
end;

end.
