unit oaBaloon;

{ Office 97 Assistant Package v1.0 }
{ Copyright  1999 Kambiz R. Khojasteh }
{ email: Khojasteh@www.dci.co.ir }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, oaTypes, OffBtn;

type

{ TBalloonOptions }

  TBalloonOptions = class(TPersistent)
  private
    FColor: TColor;
    FBorderColor: TColor;
    FFont: TFont;
    FOnChange: TNotifyEvent;
    IsFontChanged: Boolean;
    procedure SetColor(Value: TColor);
    procedure SetBorderColor(Value: TColor);
    procedure SetFont(Value: TFont);
    procedure FontChanged(Sender: TObject);
    function IsFontStored: Boolean;
  protected
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property Color: TColor read FColor write SetColor default $00C0FFFF;
    property BorderColor: TColor read FBorderColor write SetBorderColor default clBlack;
    property Font: TFont read FFont write SetFont stored IsFontStored;
  end;

{ TMsgDlgOptions }

  TMsgDlgOptions = class(TPersistent)
  private
    FTitle: String;
    FCustomIcon: TIcon;
    FOnChange: TNotifyEvent;
    procedure SetTitle(Value: String);
    procedure SetCustomIcon(Value: TIcon);
    procedure CustomIconChanged(Sender: TObject);
    function IsCustomIconStored: Boolean;
  protected
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property CustomIcon: TIcon read FCustomIcon write SetCustomIcon stored IsCustomIconStored;
    property Title: String read FTitle write SetTitle;
  end;

{ TTipsDlgOptions }

  TTipsDlgOptions = class(TPersistent)
  private
    FTips: TStringList;
    FTipIndex: Integer;
    FVisibleButtons: TTipsDlgButtons;
    FOnTipsTipChange: TTipsTipChangeEvent;
    FOnChange: TNotifyEvent;
    procedure SetVisibleButtons(Value: TTipsDlgButtons);
    procedure SetTips(Value: TStringList);
    procedure SetTipIndex(Value: Integer);
    procedure TipsChanged(Sender: TObject);
  protected
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property VisibleButtons: TTipsDlgButtons read FVisibleButtons write SetVisibleButtons
      default [Low(TTipsDlgBtn)..High(TTipsDlgBtn)];
    property Tips: TStringList read FTips write SetTips;
    property TipIndex: Integer read FTipIndex write SetTipIndex default -1;
    property OnTipsTipChange: TTipsTipChangeEvent read FOnTipsTipChange write FOnTipsTipChange;
  end;

{ TGuideDlgOptions }

  TGuideGetBtnStatesEvent = procedure(Sender: TObject;
    var OptionsBtn, SearchBtn: Boolean) of object;

  TGuideDlgOptions = class(TPersistent)
  private
    FTitle: String;
    FRequest: String;
    FTopics: TStringList;
    FDescription: String;
    FRequestBox: Boolean;
    FVisibleButtons: TGuideDlgButtons;
    FOnGuideRequestChange: TGuideRequestChangeEvent;
    FOnGuideTopicClick: TGuideTopicClickEvent;
    FOnGuideOptionsClick: TGuideOptionsClickEvent;
    FOnGuideSearchClick: TGuideSearchClickEvent;
    FOnGuideGetBtnStates: TGuideGetBtnStatesEvent;
    FOnChange: TNotifyEvent;
    InternalChange: Boolean;
    procedure SetVisibleButtons(Value: TGuideDlgButtons);
    procedure SetRequestBox(Value: Boolean);
    procedure SetTopics(Value: TStringList);
    procedure SetTitle(Value: String);
    procedure SetRequest(Value: String);
    procedure SetDescription(Value: String);
    procedure TopicsChanged(Sender: TObject);
  protected
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property VisibleButtons: TGuideDlgButtons read FVisibleButtons write SetVisibleButtons
      default [Low(TGuideDlgCmdBtn)..High(TGuideDlgCmdBtn)];
    property RequestBox: Boolean read FRequestBox write SetRequestBox default True;
    property Topics: TStringList read FTopics write SetTopics;
    property Title: String read FTitle write SetTitle;
    property Request: String read FRequest write SetRequest;
    property Description: String read FDescription write SetDescription;
    property OnGuideRequestChange: TGuideRequestChangeEvent read FOnGuideRequestChange write FOnGuideRequestChange;
    property OnGuideTopicClick: TGuideTopicClickEvent read FOnGuideTopicClick write FOnGuideTopicClick;
    property OnGuideOptionsClick: TGuideOptionsClickEvent read FOnGuideOptionsClick write FOnGuideOptionsClick;
    property OnGuideSearchClick: TGuideSearchClickEvent read FOnGuideSearchClick write FOnGuideSearchClick;
    property OnGuideGetBtnStates: TGuideGetBtnStatesEvent read FOnGuideGetBtnStates write FOnGuideGetBtnStates;
  end;

{ TAssistantBalloon }

  TBalloonShape = (bsSpeakShape, bsThinkShape);

  TAssistantBalloon = class(TForm)
    Timer: TTimer;
    MagicPanel: TPanel;
    TipIcon: TImage;
    MsgText: TLabel;
    MsgIcon: TImage;
    Title: TLabel;
    DividerTop: TShape;
    DividerBottom: TShape;
    TopicsPanel: TPanel;
    UserRequest: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure UserRequestChange(Sender: TObject);
    procedure UserRequestKeyPress(Sender: TObject; var Key: Char);
    procedure FormDeactivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FOptions: TBalloonOptions;
    FMsgDlgOptions: TMsgDlgOptions;
    FTipsDlgOptions: TTipsDlgOptions;
    FGuideDlgOptions: TGuideDlgOptions;
    FRefForm: TCustomForm;
    FBalloonKind: TBalloonKind;
    FBalloonPos: TBalloonPos;
    FInternalHide: Boolean;
    FOnBalloonShow: TNotifyEvent;
    FOnBalloonHide: TNotifyEvent;
    Modal: Boolean;
    BoxWidth: Integer;
    BoxHeight: Integer;
    BalloonShape: TBalloonShape;
    AutoHideDelay: Integer;
    BtnHeight: Integer;
    Origin: TPoint;
    ShowStep: Integer;
    procedure SetRefForm(Value: TCustomForm);
    procedure SetBalloonKind(Value: TBalloonKind);
    procedure SetInternalHide(Value: Boolean);
    procedure SetOptions(Value: TBalloonOptions);
    procedure SetMsgDlgOptions(Value: TMsgDlgOptions);
    procedure SetTipsDlgOptions(Value: TTipsDlgOptions);
    procedure SetGuideDlgOptions(Value: TGuideDlgOptions);
    function FindPos(Side: Boolean; Extra: Integer): TBalloonPos;
    function CreateSpeakRgn: HRgn;
    function CreateThinkRgn: HRgn;
    procedure CMBiDiModeChanged(var Msg: TMessage); message CM_BIDIMODECHANGED;
    procedure AutoHideDelayExpired(Sender: TObject);
    procedure SmoothShowDelayExpired(Sender: TObject);
    procedure OptionsChanged(Sender: TObject);
    procedure SetupSmoothShow;
    procedure SetupAutoHide;
    procedure Rebuild(ForceUpdate: Boolean);
  private
    MsgType: TMsgDlgType;
    MsgButtons: TMsgDlgButtons;
    MsgBtnCount: Integer;
    MsgBtnWidth: Integer;
    MsgResult: TModalResult;
    MsgBtn: array[TMsgDlgBtn] of TOffice97Button;
    procedure RebuildMsgDlg;
    procedure SelectMsgIcon;
    procedure CreateMsgButtons;
    procedure MsgBtnClick(Sender: TObject);
  private
    TipPadStr: String;
    TipsBtnCount: Integer;
    TipsBtnWidth: Integer;
    TipsBtn: array[TTipsDlgBtn] of TOffice97Button;
    procedure RebuildTipsDlg;
    procedure SelectTip;
    procedure CreateTipsButtons;
    procedure CreateTipsPadStr;
    procedure TipsBtnClick(Sender: TObject);
  private
    TopTopic: Word;
    TopicBtns: TList;
    VisibleTopics: Word;
    GuideBtnCount: Integer;
    GuideBtnWidth: Integer;
    GuideBtn: array[TGuideDlgBtn] of TOffice97Button;
    procedure DoSearch;
    procedure ShowNextPage;
    procedure ShowBackPage;
    procedure RebuildGuideDlg;
    procedure CreateGuideButtons;
    procedure GuideBtnClick(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Prepare;
    function ShowMsgDlg(const Msg: String; AType: TMsgDlgType;
      AButtons: TMsgDlgButtons; HelpCtx: LongInt): Word;
    procedure ShowMessage(const Msg: String; Timeout: Integer; Shape: TBalloonShape);
    procedure ShowTipsDlg;
    procedure ShowGuideDlg;
    property BalloonPos: TBalloonPos read FBalloonPos;
    property Options: TBalloonOptions read FOptions write SetOptions;
    property MsgDlgOptions: TMsgDlgOptions read FMsgDlgOptions write SetMsgDlgOptions;
    property TipsDlgOptions: TTipsDlgOptions read FTipsDlgOptions write SetTipsDlgOptions;
    property GuideDlgOptions: TGuideDlgOptions read FGuideDlgOptions write SetGuideDlgOptions;
    property RefForm: TCustomForm read FRefForm write SetRefForm;
    property InternalHide: Boolean read FInternalHide write SetInternalHide;
    property BalloonKind: TBalloonKind read FBalloonKind write SetBalloonKind;
    property OnBalloonShow: TNotifyEvent read FOnBalloonShow write FOnBalloonShow;
    property OnBalloonHide: TNotifyEvent read FOnBalloonHide write FOnBalloonHide;
    property IsModal: Boolean read Modal;
  end;

implementation

{$R *.DFM}

uses
  oaConst;

const
  ArrowHeight      = 17;
  ArrowWidth       = 10;
  RingsHeight      = 60;
  RoundCorner      = 15;
  RefSpace         = -12;
  MinMsgDlgWidth   = 140;
  MaxMsgDlgWidth   = 260;
  MinTipsDlgWidth  = 480;
  MaxTipsDlgWidth  = 480;
  MinGuideDlgWidth = 140;
  MaxTopicHeight   = 133;
  MinButtonWidth   = 67;
  MinButtonHeight  = 20;
  ButtonSpace      = 26;
  ControlSpace     = 7;

{ TBalloonOptions }

constructor TBalloonOptions.Create;
begin
  inherited Create;
  FColor := $00C0FFFF;
  FBorderColor := clBlack;
  FFont := TFont.Create;
  FFont.Color := clBlack;
  FFont.OnChange := FontChanged;
  IsFontChanged := False;
end;

destructor TBalloonOptions.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;

procedure TBalloonOptions.SetColor(Value: TColor);
begin
  if FColor <> Value then
  begin
    FColor := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TBalloonOptions.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TBalloonOptions.SetFont(Value: TFont);
begin
  if (FFont <> Value) and Assigned(Value) then
    FFont := Value;
end;

procedure TBalloonOptions.FontChanged(Sender: TObject);
begin
  IsFontChanged := True;
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

function TBalloonOptions.IsFontStored: Boolean;
begin
  Result := IsFontChanged;
end;

{ TMsgDlgBtns }

const
  MsgBtnResult: array[TMsgDlgBtn] of TModalResult =
    (mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry,
     mrIgnore, mrAll, mrNoToAll, mrYesToAll, mrNone);

{ TMsgDlgOptions }

constructor TMsgDlgOptions.Create;
begin
  inherited Create;
  FTitle := EmptyStr;
  FCustomIcon := TIcon.Create;
  FCustomIcon.OnChange := CustomIconChanged;
end;

destructor TMsgDlgOptions.Destroy;
begin
  FCustomIcon.Free;
  inherited Destroy;
end;

procedure TMsgDlgOptions.SetTitle(Value: String);
begin
  if FTitle <> Value then
  begin
    FTitle := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TMsgDlgOptions.SetCustomIcon(Value: TIcon);
begin
  if Value <> nil then
  begin
    if FCustomIcon.Handle <> Value.Handle then
      FCustomIcon.Assign(Value);
  end
  else if not FCustomIcon.Empty then
    FCustomIcon.ReleaseHandle;
end;

procedure TMsgDlgOptions.CustomIconChanged(Sender: TObject);
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

function TMsgDlgOptions.IsCustomIconStored: Boolean;
begin
  Result := not FCustomIcon.Empty;
end;

{ TTipsDlgOptions }

constructor TTipsDlgOptions.Create;
begin
  inherited Create;
  FVisibleButtons := [Low(TTipsDlgBtn)..High(TTipsDlgBtn)];
  FTips := TStringList.Create;
  FTips.OnChange := TipsChanged;
  FTipIndex := -1;
end;

destructor TTipsDlgOptions.Destroy;
begin
  FTips.Free;
  inherited Destroy;
end;

procedure TTipsDlgOptions.SetTips(Value: TStringList);
begin
  if FTips <> Value then
  begin
    if Value = nil then
      FTips.Clear
    else
      FTips.Assign(Value);
  end;
end;

procedure TTipsDlgOptions.SetTipIndex(Value: Integer);
begin
  if Value < 0 then
    Value := 0;
  if Value >= FTips.Count then
    Value := FTips.Count-1;
  if FTipIndex <> Value then
  begin
    FTipIndex := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TTipsDlgOptions.SetVisibleButtons(Value: TTipsDlgButtons);
begin
  if FVisibleButtons <> Value then
  begin
    FVisibleButtons := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TTipsDlgOptions.TipsChanged(Sender: TObject);
begin
  if FTipIndex >= FTips.Count then
    FTipIndex := FTips.Count-1
  else if (FTipIndex = -1) and (FTips.Count > 0) then
    FTipIndex := 0;
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

{ TGuideDlgOptions }

constructor TGuideDlgOptions.Create;
begin
  inherited Create;
  FVisibleButtons := [Low(TGuideDlgCmdBtn)..High(TGuideDlgCmdBtn)];
  FTopics := TStringList.Create;
  FTopics.OnChange := TopicsChanged;
  FTitle := EmptyStr;
  FRequest := EmptyStr;
  FDescription := EmptyStr;
  FRequestBox := True;
  InternalChange := False;
end;

destructor TGuideDlgOptions.Destroy;
begin
  FTopics.Free;
  inherited Destroy;
end;

procedure TGuideDlgOptions.SetTopics(Value: TStringList);
begin
  if FTopics <> Value then
  begin
    if Value = nil then
      FTopics.Clear
    else
      FTopics.Assign(Value);
  end;
end;

procedure TGuideDlgOptions.SetVisibleButtons(Value: TGuideDlgButtons);
begin
  if FVisibleButtons <> Value then
  begin
    FVisibleButtons := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TGuideDlgOptions.SetTitle(Value: String);
begin
  if FTitle <> Value then
  begin
    FTitle := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TGuideDlgOptions.SetRequest(Value: String);
begin
  if FRequest <> Value then
  begin
    FRequest := Value;
    if not InternalChange and Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TGuideDlgOptions.SetRequestBox(Value: Boolean);
begin
  if FRequestBox <> Value then
  begin
    FRequestBox := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TGuideDlgOptions.SetDescription(Value: String);
begin
  if FDescription <> Value then
  begin
    FDescription := Value;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TGuideDlgOptions.TopicsChanged(Sender: TObject);
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

{ TAssistantBalloon }

constructor TAssistantBalloon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOptions := TBalloonOptions.Create;
  FMsgDlgOptions := TMsgDlgOptions.Create;
  FTipsDlgOptions := TTipsDlgOptions.Create;
  FGuideDlgOptions := TGuideDlgOptions.Create;
  FBalloonKind := bkNone;
  FRefForm := nil;
  FInternalHide := False;
  Modal := False;
  TopicBtns := TList.Create;
  BoxWidth := 200;
  BoxHeight := 100;
  AutoHideDelay := 0;
  BalloonShape := bsSpeakShape;
end;

destructor TAssistantBalloon.Destroy;
begin
  TopicBtns.Free;
  FOptions.Free;
  FMsgDlgOptions.Free;
  FTipsDlgOptions.Free;
  FGuideDlgOptions.Free;
  inherited Destroy;
end;

procedure TAssistantBalloon.SetRefForm(Value: TCustomForm);
begin
  if FRefForm <> Value then
  begin
    FRefForm := Value;
    if Visible then Prepare;
  end;
end;

procedure TAssistantBalloon.SetInternalHide(Value: Boolean);
begin
  if FInternalHide <> Value then
  begin
    if Value then
    begin
      if Visible then
      begin
        FInternalHide := True;
        Hide;
      end;
    end
    else
    begin
      if not Visible then
        Show
      else
        FInternalHide := False;
    end;
  end;
end;

procedure TAssistantBalloon.SetOptions(Value: TBalloonOptions);
begin
  if (FOptions <> Value) and Assigned(Value) then
  begin
    FOptions.Assign(Value);
    OptionsChanged(FOptions);
  end;
end;

procedure TAssistantBalloon.SetMsgDlgOptions(Value: TMsgDlgOptions);
begin
  if (FMsgDlgOptions <> Value) and Assigned(Value) then
  begin
    FMsgDlgOptions.Assign(Value);
    OptionsChanged(FMsgDlgOptions);
  end;
end;

procedure TAssistantBalloon.SetTipsDlgOptions(Value: TTipsDlgOptions);
begin
  if (FTipsDlgOptions <> Value) and Assigned(Value) then
  begin
    FTipsDlgOptions.Assign(Value);
    OptionsChanged(FTipsDlgOptions);
  end;
end;

procedure TAssistantBalloon.SetGuideDlgOptions(Value: TGuideDlgOptions);
begin
  if (FGuideDlgOptions <> Value) and Assigned(Value) then
  begin
    FGuideDlgOptions.Assign(Value);
    OptionsChanged(FGuideDlgOptions);
  end;
end;

procedure TAssistantBalloon.Prepare;
var
  Rgn: HRgn;
  Bound: TRect;
  OrgX, OrgY, Extra: Integer;
begin
  OrgX := Origin.X;
  OrgY := Origin.Y;
  if BalloonShape = bsSpeakShape then
    Extra := ArrowHeight
  else
    Extra := RingsHeight;
  FBalloonPos := FindPos(BalloonKind = bkTips, Extra);
  case FBalloonPos of
    bpTopRight:
      SetRect(Bound, FRefForm.Left + FRefForm.Width - BoxWidth, FRefForm.Top -
        (BoxHeight + Extra) - RefSpace, BoxWidth, BoxHeight + Extra);
    bpTopLeft:
      SetRect(Bound, FRefForm.Left, FRefForm.Top - (BoxHeight + Extra) -
        RefSpace, BoxWidth, BoxHeight + Extra);
    bpRightTop:
      SetRect(Bound, FRefForm.Left + FRefForm.Width + RefSpace, FRefForm.Top,
        BoxWidth + Extra, BoxHeight);
    bpRightBottom:
      SetRect(Bound, FRefForm.Left + FRefForm.Width + RefSpace, FRefForm.Top +
        FRefForm.Height - BoxHeight, BoxWidth + Extra, BoxHeight);
    bpBottomLeft:
      SetRect(Bound, FRefForm.Left, FRefForm.Top + FRefForm.Height + RefSpace,
        BoxWidth, BoxHeight + Extra);
    bpBottomRight:
      SetRect(Bound, FRefForm.Left + FRefForm.Width - BoxWidth, FRefForm.Top +
        FRefForm.Height + RefSpace, BoxWidth, BoxHeight + Extra);
    bpLeftBottom:
      SetRect(Bound, FRefForm.Left - (BoxWidth + Extra) - RefSpace, FRefForm.Top +
        FRefForm.Height - BoxHeight, BoxWidth + Extra, BoxHeight);
    bpLeftTop:
      SetRect(Bound, FRefForm.Left - (BoxWidth + Extra) - RefSpace,
        FRefForm.Top, BoxWidth + Extra, BoxHeight);
    bpCenter:
      SetRect(Bound, (Screen.Width - BoxWidth) div 2, (Screen.Height -
        BoxHeight) div 2, BoxWidth, BoxHeight);
  end;
  Inc(Bound.Right, Bound.Left);
  Inc(Bound.Bottom, Bound.Top);
  if Bound.Left <= 0 then
    OffsetRect(Bound, 1-Bound.Left, 0);
  if Bound.Right > Screen.DesktopWidth then
    OffsetRect(Bound, Screen.DesktopWidth - Bound.Right, 0);
  if Bound.Top <= 0 then
    OffsetRect(Bound, 0, 1-Bound.Top);
  if Bound.Bottom > Screen.DesktopHeight then
    OffsetRect(Bound, 0, Screen.DesktopHeight - Bound.Bottom);
  Origin := Point(0,0);
  if FBalloonPos in [bpBottomLeft, bpBottomRight] then
    Inc(Origin.Y, Extra)
  else if FBalloonPos in [bpRightTop, bpRightBottom] then
    Inc(Origin.X, Extra);
  BoundsRect := Bound;
  ScrollBy(Origin.X - OrgX, Origin.Y - OrgY);
  if BalloonShape = bsSpeakShape then
    Rgn := CreateSpeakRgn
  else
    Rgn := CreateThinkRgn;
  SetWindowRgn(Handle, Rgn, Visible);
  if Visible then Refresh;
end;

function TAssistantBalloon.FindPos(Side: Boolean; Extra: Integer): TBalloonPos;
var
  Space, ExSpace: TRect;
  MaxV, MaxH: Integer;

  function FindHPos: TBalloonPos;
  var
    V, H: TBalloonPos;
  begin
    V := bpTopLeft;
    if ((ExSpace.Top > ExSpace.Bottom) and (ExSpace.Top < BoxHeight)) or (FRefForm.Top < 0) then
      V := bpBottomLeft;
    if UseRightToLeftAlignment then
    begin
      H := bpLeftTop;
      if (Space.Left < Space.Right) and (Space.Left < (BoxWidth + Extra)) then
        H := bpRightTop;
    end
    else
    begin
      H := bpRightTop;
      if (Space.Right < Space.Left) and (Space.Right < (BoxWidth + Extra)) then
        H := bpLeftTop;
    end;
    if V = bpTopLeft then
      if H = bpLeftTop then
        Result := bpLeftTop
      else
        Result := bpRightTop
    else
      if H = bpLeftTop then
        Result := bpLeftBottom
      else
        Result := bpRightBottom;
  end;

  function FindVPos: TBalloonPos;
  var
    V, H: TBalloonPos;
  begin
    if UseRightToLeftAlignment then
    begin
      H := bpRightTop;
      if (ExSpace.Right > ExSpace.Left) and (ExSpace.Left < BoxWidth) then
        H := bpLeftTop;
    end
    else
    begin
      H := bpLeftTop;
      if (ExSpace.Left > ExSpace.Right) and (ExSpace.Right < BoxWidth) then
        H := bpRightTop;
    end;
    V := bpTopLeft;
    if (Space.Top < Space.Bottom) and (Space.Top < (BoxHeight + Extra)) then
      V := bpBottomLeft;
    if V = bpTopLeft then
      if H = bpLeftTop then
        Result := bpTopLeft
      else
        Result := bpTopRight
    else
      if H = bpLeftTop then
        Result := bpBottomLeft
      else
        Result := bpBottomRight;
  end;

begin
  if not (Assigned(FRefForm) and FRefForm.Visible) then
    Result := bpCenter
  else
  begin
    Space.Top := FRefForm.Top;
    Space.Bottom := Screen.DesktopHeight - (FRefForm.Top + FRefForm.Height);
    Space.Left := FRefForm.Left;
    Space.Right := Screen.DesktopWidth - (FRefForm.Left + FRefForm.Width);
    ExSpace.Top := Space.Top + FRefForm.Height;
    ExSpace.Bottom := Space.Bottom + FRefForm.Height;
    ExSpace.Left := Space.Left + FRefForm.Width;
    ExSpace.Right := Space.Right + FRefForm.Width;
    if Space.Right >= Space.Left then
      MaxH := Space.Right
    else
      MaxH := Space.Left;
    if Space.Top >= Space.Bottom then
      MaxV := Space.Top
    else
      MaxV := Space.Bottom;
    Side := Side or (Space.Top < (BoxHeight + Extra));
    if (Side and (MaxH >= (BoxWidth + Extra))) or (MaxV < (BoxHeight + Extra)) then
      Result := FindHPos
    else
      Result := FindVPos;
  end;
end;

function TAssistantBalloon.CreateSpeakRgn: HRgn;
var
 AP: array[0..2] of TPoint;
 Offset: Integer;
 Rgn: HRgn;
begin
  if FBalloonPos in [bpTopLeft, bpTopRight, bpBottomLeft, bpBottomRight] then
  begin
    if Width < FRefForm.Width then
      Offset := 3 * Width div 4
    else
      Offset := 2 * RefForm.Width div 3;
    if FRefForm.Left < 0 then
      Inc(Offset, FRefForm.Left);
    if (FRefForm.Left + FRefForm.Width) > Screen.DesktopWidth then
      Dec(Offset, FRefForm.Left + FRefForm.Width - Screen.DesktopWidth);
    if Offset < RoundCorner then
      Offset := RoundCorner;
  end
  else
  begin
    Offset := 40;
    if (Top < FRefForm.Top) and (FBalloonPos in [bpLeftTop, bpRightTop]) then
      Inc(Offset, RefForm.Top - Top)
    else if ((Top + Height) > (FRefForm.Top + FRefForm.Height)) and
      (FBalloonPos in [bpLeftBottom, bpRightBottom]) then
      Inc(Offset, (Top + Height) - (FRefForm.Top + FRefForm.Height));
    if FRefForm.Top < 0 then
      Dec(Offset, FRefForm.Top);
    if (FRefForm.Top + FRefForm.Height) > Screen.DesktopHeight then
      Inc(Offset, FRefForm.Top + FRefForm.Height - Screen.DesktopHeight);
    if Offset > (Height - 2 * RoundCorner) then
      Offset := Height - 2 * RoundCorner;
  end;
  case FBalloonPos of
    bpTopLeft:
    begin
      AP[0] := Point(Offset, Height-ArrowHeight-1);
      AP[1] := Point(Offset+ArrowWidth, Height-ArrowHeight-1);
      AP[2] := Point(Offset, Height-1);
    end;
    bpTopRight:
    begin
      AP[0] := Point(Width-Offset-ArrowWidth-1, Height-ArrowHeight-1);
      AP[1] := Point(Width-Offset-1, Height-ArrowHeight-1);
      AP[2] := Point(Width-Offset-1, Height-1);
    end;
    bpBottomLeft:
    begin
      AP[0] := Point(Offset, ArrowHeight);
      AP[1] := Point(Offset+ArrowWidth, ArrowHeight);
      AP[2] := Point(Offset, 0);
    end;
    bpBottomRight:
    begin
      AP[0] := Point(Width-Offset-ArrowWidth-1, ArrowHeight);
      AP[1] := Point(Width-Offset-1, ArrowHeight);
      AP[2] := Point(Width-Offset-1, 0);
    end;
    bpRightTop:
    begin
      AP[0] := Point(ArrowHeight, Offset);
      AP[1] := Point(ArrowHeight, Offset+ArrowWidth);
      AP[2] := Point(0, Offset);
    end;
    bpRightBottom:
    begin
      AP[0] := Point(ArrowHeight, Height-Offset-1);
      AP[1] := Point(ArrowHeight, Height-Offset+ArrowWidth-1);
      AP[2] := Point(0, Height-Offset-1);
    end;
    bpLeftTop:
    begin
      AP[0] := Point(Width-ArrowHeight-1, Offset);
      AP[1] := Point(Width-ArrowHeight-1, Offset+ArrowWidth);
      AP[2] := Point(Width-1, Offset);
    end;
    bpLeftBottom:
    begin
      AP[0] := Point(Width-ArrowHeight-1, Height-Offset-1);
      AP[1] := Point(Width-ArrowHeight-1, Height-Offset+ArrowWidth-1);
      AP[2] := Point(Width-1, Height-Offset-1);
    end;
  end;
  Result := CreateRoundRectRgn(Origin.X, Origin.Y, Origin.X + BoxWidth,
    Origin.Y + BoxHeight, RoundCorner, RoundCorner);
  if FBalloonPos <> bpCenter then
  begin
    Rgn := CreatePolygonRgn(AP, 3, WINDING);
    CombineRgn(Result, Result, Rgn, RGN_OR);
    DeleteObject(Rgn);
  end;
end;

function TAssistantBalloon.CreateThinkRgn: HRgn;
var
 Offset1, Offset2, Radius: Integer;
 I, X, Y: Integer;
 Rgns: array[1..3] of HRgn;
begin
  if FBalloonPos in [bpTopLeft, bpTopRight, bpBottomLeft, bpBottomRight] then
  begin
    Offset1 := RefForm.Width div 2 + 15;
    if FRefForm.Left < 0 then
      Inc(Offset1, FRefForm.Left);
    if (FRefForm.Left + FRefForm.Width) > Screen.DesktopWidth then
      Dec(Offset1, FRefForm.Left + FRefForm.Width - Screen.DesktopWidth);
    if Offset1 < RoundCorner then
      Offset1 := RoundCorner;
    Offset2 := BoxWidth div 2;
  end
  else
  begin
    Offset1 := 30;
    if FRefForm.Top < 0 then
      Dec(Offset1, FRefForm.Top);
    if (FRefForm.Top + FRefForm.Height) > Screen.DesktopHeight then
      Inc(Offset1, FRefForm.Top + FRefForm.Height - Screen.DesktopHeight);
    if Offset1 > (Height - RoundCorner) then
      Offset1 := Height - RoundCorner;
    Offset2 := BoxHeight div 2;
  end;
  case FBalloonPos of
    bpTopLeft:
      for I := 1 to 3 do
      begin
        Y := Height - (2 * I - 1) * RingsHeight div 6 + 2 * I;
        X := OffSet1 + (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpTopRight:
      for I := 1 to 3 do
      begin
        Y := Height - (2 * I - 1) * RingsHeight div 6 + 2 * I;
        X := Width - OffSet1 - (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpBottomLeft:
      for I := 1 to 3 do
      begin
        Y := (2 * I - 1) * RingsHeight div 6 - 2 * I;
        X := OffSet1 + (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpBottomRight:
      for I := 1 to 3 do
      begin
        Y := (2 * I - 1) * RingsHeight div 6 - 2 * I;
        X := Width - OffSet1 - (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpLeftTop:
      for I := 1 to 3 do
      begin
        X := Width - (2 * I - 1) * RingsHeight div 6 + 2 * I;
        Y := OffSet1 + (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpLeftBottom:
      for I := 1 to 3 do
      begin
        X := Width - (2 * I - 1) * RingsHeight div 6 + 2 * I;
        Y := Height - OffSet1 - (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpRightTop:
      for I := 1 to 3 do
      begin
        X := (2 * I - 1) * RingsHeight div 6 - 2 * I;
        Y := OffSet1 + (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
    bpRightBottom:
      for I := 1 to 3 do
      begin
        X := (2 * I - 1) * RingsHeight div 6 - 2 * I;
        Y := Height - OffSet1 - (I - 1) * (Offset2 - Offset1) div 3;
        Radius := (RingsHeight div 7) + 2 * I - 5;
        Rgns[I] := CreateEllipticRgn(X - Radius, Y - Radius, X + Radius, Y + Radius);
      end;
  end;
  Result := CreateRoundRectRgn(Origin.X, Origin.Y, Origin.X + BoxWidth,
    Origin.Y + BoxHeight, RoundCorner, RoundCorner);
  if FBalloonPos <> bpCenter then
    for I := 1 to 3 do
    begin
      CombineRgn(Result, Result, Rgns[I], RGN_OR);
      DeleteObject(Rgns[I]);
    end;
end;

procedure TAssistantBalloon.SetupSmoothShow;
begin
  if MagicPanel.Visible then
  begin
    DoubleBuffered := False;
    MagicPanel.Color := Options.BorderColor;
    MagicPanel.BringToFront;
    ShowStep := 10;
    Timer.Interval := 30;
    Timer.OnTimer := SmoothShowDelayExpired;
    Timer.Enabled := True;
    SmoothShowDelayExpired(Self);
  end;
end;

procedure TAssistantBalloon.SetupAutoHide;
begin
  if AutoHideDelay <> 0 then
  begin
    Timer.Interval := AutoHideDelay;
    Timer.OnTimer := AutoHideDelayExpired;
    Timer.Enabled := True;
  end;
end;

procedure TAssistantBalloon.CMBiDiModeChanged(var Msg: TMessage);
begin
  inherited;
  Rebuild(False);
end;

procedure TAssistantBalloon.SmoothShowDelayExpired(Sender: TObject);
var
  R: TRect;
  Rgn1, Rgn2: HRgn;
begin
  ShowStep := 2 * ShowStep;
  if ShowStep < 50 then
  begin
    SetRect(R, BoxWidth div 2,  BoxHeight div 2, BoxWidth div 2, BoxHeight div 2);
    InflateRect(R, BoxWidth * ShowStep div 100, BoxHeight * ShowStep div 100);
    OffsetRect(R, Origin.X, Origin.Y);
    Rgn1 := CreateRectRgnIndirect(R);
    InflateRect(R, -1, -1);
    Rgn2 := CreateRectRgnIndirect(R);
    CombineRgn(Rgn1, Rgn2, Rgn1, RGN_XOR);
    DeleteObject(Rgn2);
    SetWindowRgn(Handle, Rgn1, True);
    MagicPanel.Update;
  end
  else
  begin
    Timer.Enabled := False;
    MagicPanel.Visible := False;
    if BalloonShape = bsSpeakShape then
      Rgn1 := CreateSpeakRgn
    else
      Rgn1 := CreateThinkRgn;
    DoubleBuffered := True;
    SetWindowRgn(Handle, Rgn1, True);
    Update;
    BringToFront;
    SetupAutoHide;
    if not FInternalHide and Assigned(FOnBalloonShow) then
      FOnBalloonShow(Self);
    FInternalHide := False;
  end;
end;

procedure TAssistantBalloon.AutoHideDelayExpired(Sender: TObject);
begin
  FInternalHide := False;
  Timer.Enabled := False;
  Close;
end;

procedure TAssistantBalloon.FormCreate(Sender: TObject);
begin
  FOptions.OnChange := OptionsChanged;
  FMsgDlgOptions.OnChange := OptionsChanged;
  FTipsDlgOptions.OnChange := OptionsChanged;
  FGuideDlgOptions.OnChange := OptionsChanged;
  OptionsChanged(FOptions);
end;

procedure TAssistantBalloon.FormPaint(Sender: TObject);
var
  Rgn: HRgn;
begin
  if not MagicPanel.Visible then
  begin
    if BalloonShape = bsSpeakShape then
      Rgn := CreateSpeakRgn
    else
      Rgn := CreateThinkRgn;
    try
      Canvas.Brush.Color := Options.BorderColor;
      FrameRgn(Canvas.Handle, Rgn, Canvas.Brush.Handle, 1, 1);
      Canvas.Brush.Color := Options.Color;
    finally
      DeleteObject(Rgn);
    end;
  end;
end;

procedure TAssistantBalloon.FormShow(Sender: TObject);
begin
  Prepare;
  BringToFront;
  SetupSmoothShow;
end;

procedure TAssistantBalloon.FormHide(Sender: TObject);
begin
  if not FInternalHide then
    Modal := False;
  Timer.Enabled := False;
  MagicPanel.Visible := True;
  if not FInternalHide and Assigned(FOnBalloonHide) then
    FOnBalloonHide(Self);
end;

procedure TAssistantBalloon.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if ((BalloonKind <> bkMessage) or (MsgBtnCount = 0)) and (Key = Chr(VK_ESCAPE)) then
    Close;
end;

procedure TAssistantBalloon.FormDeactivate(Sender: TObject);
begin
  if Modal and (Screen.ActiveForm <> FRefForm) then
  begin
    SetForegroundWindow(Handle);
    MessageBeep(0);
  end;
end;

procedure TAssistantBalloon.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  B: TMsgDlgBtn;
begin
  if Modal and not FInternalHide and (MsgResult = mrNone) then
  begin
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if MsgBtn[B].Cancel then
      begin
        MsgResult := MsgBtnResult[B];
        Break;
      end;
    CanClose := (MsgResult <> mrNone);
  end;
end;

procedure TAssistantBalloon.OptionsChanged(Sender: TObject);
var
  TextHeight: Integer;
begin
  if csDestroying in ComponentState then Exit;
  if Sender is TBalloonOptions then
  begin
    Color := Options.Color;
    Font := Options.Font;
    TextHeight := Canvas.TextHeight('H');
    BtnHeight := TextHeight + 10;
    if BtnHeight < MinButtonHeight then
      BtnHeight := MinButtonHeight;
    Title.Font := Font;
    Title.Font.Style := Font.Style + [fsBold];
    if Assigned(GuideBtn[gbSearch]) then
      GuideBtn[gbSearch].Font.Style := Font.Style + [fsBold];
    UserRequest.Height := 2 * (TextHeight + 2);
    CreateTipsPadStr;
    CreateMsgButtons;
    CreateTipsButtons;
    CreateGuideButtons;
    Rebuild(False);
  end
  else if ((Sender is TMsgDlgOptions) and (BalloonKind in [bkMessage, bkSpeak, bkThink])) or
          ((Sender is TTipsDlgOptions) and (BalloonKind = bkTips)) or
          ((Sender is TGuideDlgOptions) and (BalloonKind = bkGuide)) then
    Rebuild(False);
end;

procedure TAssistantBalloon.SetBalloonKind(Value: TBalloonKind);
var
  MB: TMsgDlgBtn;
  TB: TTipsDlgBtn;
  GB: TGuideDlgBtn;
begin
  if Visible then Hide;
  if BalloonKind <> Value then
  begin
    FBalloonKind := Value;
    MsgIcon.Visible := (BalloonKind = bkMessage);
    for MB := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if Assigned(MsgBtn[MB]) then
        MsgBtn[MB].Visible := (BalloonKind = bkMessage);
    TipIcon.Visible := (BalloonKind = bkTips);
    for TB := Low(TTipsDlgBtn) to High(TTipsDlgBtn) do
      if Assigned(TipsBtn[TB]) then
        TipsBtn[TB].Visible := (BalloonKind = bkTips);
    TopicsPanel.Visible := (BalloonKind = bkGuide);
    UserRequest.Visible := (BalloonKind = bkGuide);
    for GB := Low(TGuideDlgCmdBtn) to High(TGuideDlgCmdBtn) do
      if Assigned(GuideBtn[GB]) then
        GuideBtn[GB].Visible := (BalloonKind = bkGuide);
    DividerTop.Visible := (BalloonKind = bkGuide);
    DividerBottom.Visible := ((BalloonKind = bkMessage) and (MsgBtnCount > 0)) or
      ((BalloonKind = bkGuide) and (GuideBtnCount > 0));
    Title.Visible := (BalloonKind in [bkMessage, bkGuide]);
    Rebuild(False);
  end;
end;

procedure TAssistantBalloon.Rebuild(ForceUpdate: Boolean);
begin
  if Visible or ForceUpdate then
  begin
    case BalloonKind of
      bkSpeak, bkThink, bkMessage:RebuildMsgDlg;
      bkTips: RebuildTipsDlg;
      bkGuide: RebuildGuideDlg;
    end;
    if Visible then Prepare;
  end;
end;

procedure SetFocusToFirstControlOf(Form: TCustomForm);
var
  I: Integer;
  Control, ThisControl: TWinControl;
begin
  if Assigned(Form) and Form.Visible and ((Form.ActiveControl = nil) or
    not Form.ActiveControl.Visible or not Form.ActiveControl.Enabled) then
  begin
    Control := nil;
    for I := 0 to Form.ControlCount-1 do
      if Form.Controls[I] is TWinControl then
      begin
        ThisControl := Form.Controls[I] as TWinControl;
        if ThisControl.Visible and ThisControl.Enabled and
          (not Assigned(Control) or (ThisControl.TabOrder < Control.TabOrder)) then
        begin
          Control := ThisControl;
          if Control.TabOrder = 0 then Break;
        end;
      end;
    if Assigned(Control) then
      Form.ActiveControl := Control;
  end;
end;

function TAssistantBalloon.ShowMsgDlg(const Msg: String; AType: TMsgDlgType;
  AButtons: TMsgDlgButtons; HelpCtx: LongInt): Word;
begin
  if Visible then Hide;
  Origin := Point(0,0);
  BalloonKind := bkMessage;
  AutoHideDelay := 0;
  BalloonShape := bsSpeakShape;
  HelpContext := HelpCtx;
  Title.Caption := MsgDlgOptions.Title;
  MsgText.Caption := Msg;
  MsgType := AType;
  SelectMsgIcon;
  MsgButtons := AButtons;
  CreateMsgButtons;
  Rebuild(True);
  MsgResult := mrNone;
  // round arround TOffice97Button bug
  if MsgBtnCount <> 0 then
    SetFocusToFirstControlOf(Application.MainForm);
  // end of - round arround TOffice97Button bug
  Show;
  if MsgBtnCount <> 0 then
  begin
    Modal := True;
    while (IsModal and (MsgResult = mrNone)) or InternalHide do
      Application.ProcessMessages;
    Modal := False;
    // round arround TOffice97Button bug
    if Assigned(Application.MainForm) then
      Windows.SetFocus(Application.MainForm.Handle);
    Application.ProcessMessages;
    // end of - round arround TOffice97Button bug
    Close;
  end;
  Result := MsgResult;
end;

procedure TAssistantBalloon.ShowMessage(const Msg: String; Timeout: Integer;
  Shape: TBalloonShape);
var                      
  ActiveForm: TCustomForm;
begin
  if Visible then Hide;
  Origin := Point(0,0);
  if Shape = bsSpeakShape then
    BalloonKind := bkSpeak
  else
    BalloonKind := bkThink;
  AutoHideDelay := Timeout;
  BalloonShape := Shape;
  Title.Caption := EmptyStr;
  MsgText.Caption := Msg;
  MsgIcon.Picture.Graphic := nil;
  MsgButtons := [];
  CreateMsgButtons;
  HelpContext := 0;
  Rebuild(True);
  ActiveForm := Screen.ActiveCustomForm;
  Show;
  SetForegroundWindow(ActiveForm.Handle);
end;

procedure TAssistantBalloon.RebuildMsgDlg;
var
  R: TRect;
  B: TMsgDlgBtn;
  BtnsWidth: Integer;
  ButtonLeft, ButtonTop: Integer;
begin
  BtnsWidth := MsgBtnCount * (MsgBtnWidth + ControlSpace) - ControlSpace;
  SetRect(R, 0, 0, MaxMsgDlgWidth - 2 * ControlSpace, 0);
  if R.Right < BtnsWidth then R.Right := BtnsWidth;
  if R.Right < Title.Width + ControlSpace + MsgIcon.Width then
    R.Right := Title.Width + ControlSpace + MsgIcon.Width;
  DrawText(Canvas.Handle, PChar(MsgText.Caption), Length(MsgText.Caption), R,
    DrawTextBiDiModeFlags(DT_NOPREFIX or DT_WORDBREAK or DT_CALCRECT));
  if R.Right < (MinMsgDlgWidth - 2 * ControlSpace) then
    R.Right := MinMsgDlgWidth - 2 * ControlSpace;
  if R.Right < BtnsWidth then R.Right := BtnsWidth;
  if R.Right < Title.Width + ControlSpace + MsgIcon.Width then
    R.Right := Title.Width + ControlSpace + MsgIcon.Width;
  if (MsgIcon.Picture.Graphic = nil) then
    MsgIcon.Top := Origin.Y + ControlSpace + Title.Height
  else if MsgIcon.Height < Title.Height then
    MsgIcon.Top := Origin.Y + ControlSpace + Title.Height - MsgIcon.Height
  else
    MsgIcon.Top := Origin.Y + ControlSpace;
  Title.Top := MsgIcon.Top + MsgIcon.Height - Title.Height;
  OffsetRect(R, Origin.X + ControlSpace, MsgIcon.Top + MsgIcon.Height + 2 * ControlSpace);
  if (MsgIcon.Picture.Graphic = nil) and (Title.Caption = EmptyStr) then
    OffsetRect(R, 0, -(Title.Height + ControlSpace));
  MsgText.BoundsRect := R;
  DividerBottom.Top := R.Bottom + ControlSpace;
  DividerBottom.Width := MsgText.Width;
  DividerBottom.Left := R.Left;
  ButtonTop := DividerBottom.Top + ControlSpace - 2;
  ButtonLeft := Origin.X + ControlSpace + (MsgText.Width - BtnsWidth) div 2;
  if UseRightToLeftAlignment then
  begin
    MsgIcon.Left := R.Right - MsgIcon.Width;
    if MsgIcon.Picture.Graphic <> nil then
      Title.Left := MsgIcon.Left - Title.Width - ControlSpace
    else
      Title.Left := R.Right - Title.Width;
    for B := High(TMsgDlgBtn) downto Low(TMsgDlgBtn) do
      if Assigned(MsgBtn[B]) then
      begin
        MsgBtn[B].Left := ButtonLeft;
        MsgBtn[B].Top := ButtonTop;
        Inc(ButtonLeft, MsgBtnWidth + ControlSpace);
      end;
  end
  else
  begin
    MsgIcon.Left := R.Left;
    if MsgIcon.Picture.Graphic <> nil then
      Title.Left := MsgIcon.Left + MsgIcon.Width + ControlSpace
    else
      Title.Left := R.Left;
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if Assigned(MsgBtn[B]) then
      begin
        MsgBtn[B].Left := ButtonLeft;
        MsgBtn[B].Top := ButtonTop;
        Inc(ButtonLeft, MsgBtnWidth + ControlSpace);
      end;
  end;
  DividerBottom.Visible := MsgBtnCount > 0;
  BoxWidth := MsgText.Width + 2 * ControlSpace;
  if MsgBtnCount = 0 then
    BoxHeight := DividerBottom.Top + ControlSpace - Origin.Y
  else
    BoxHeight := ButtonTop + BtnHeight + ControlSpace - Origin.Y;
end;

procedure TAssistantBalloon.SelectMsgIcon;
begin
  if MsgType <> mtCustom then
  begin
    if MsgIcon.Picture.Graphic = nil then
      MsgIcon.Picture.Graphic := TIcon.Create;
    case MsgType of
      mtWarning: MsgIcon.Picture.Icon.Handle := LoadIcon(0, IDI_EXCLAMATION);
      mtError: MsgIcon.Picture.Icon.Handle := LoadIcon(0, IDI_HAND);
      mtInformation: MsgIcon.Picture.Icon.Handle := LoadIcon(0, IDI_ASTERISK);
      mtConfirmation: MsgIcon.Picture.Icon.Handle := LoadIcon(0, IDI_QUESTION);
    end;
  end
  else
    MsgIcon.Picture.Assign(MsgDlgOptions.CustomIcon);
end;

procedure TAssistantBalloon.CreateMsgButtons;
var
  ThisBtnWidth: Integer;
  B, DefaultButton, CancelButton: TMsgDlgBtn;
begin
  MsgBtnCount := 0;
  MsgBtnWidth := MinButtonWidth;
  for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
    if B in MsgButtons then
    begin
      Inc(MsgBtnCount);
      ThisBtnWidth := Canvas.TextWidth(DefMsgDlgBtns[B]) + ButtonSpace;
      if MsgBtnWidth < ThisBtnWidth then MsgBtnWidth := ThisBtnWidth;
    end;
  if mbOk in MsgButtons then DefaultButton := mbOk else
    if mbYes in MsgButtons then DefaultButton := mbYes else
      DefaultButton := mbRetry;
  if mbCancel in MsgButtons then CancelButton := mbCancel else
    if mbNo in MsgButtons then CancelButton := mbNo else
      CancelButton := mbOk;
  for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
    if B in MsgButtons then
    begin
      If not Assigned(MsgBtn[B]) then
      begin
        MsgBtn[B]:= TOffice97Button.Create(Self);
        MsgBtn[B].Parent := Self;
        MsgBtn[B].Tag := Ord(B);
        MsgBtn[B].WordWrap := False;
        MsgBtn[B].OnClick := MsgBtnClick;
      end;
      MsgBtn[B].Caption := DefMsgDlgBtns[B];
      MsgBtn[B].Width := MsgBtnWidth;
      MsgBtn[B].Height := BtnHeight;
      MsgBtn[B].Visible := (BalloonKind = bkMessage);
      MsgBtn[B].Color := Options.Color;
      MsgBtn[B].Cancel := (B = CancelButton);
      if MsgBtn[B].Visible and (B = DefaultButton) then
        ActiveControl := MsgBtn[B];
      if B = mbHelp then
        MsgBtn[B].Enabled := (HelpContext <> 0);
    end
    else if Assigned(MsgBtn[B]) then
    begin
      MsgBtn[B].Free;
      MsgBtn[B] := nil;
    end;
end;

procedure TAssistantBalloon.MsgBtnClick(Sender: TObject);
var
  B: TMsgDlgBtn;
begin
  B := TMsgDlgBtn(TOffice97Button(Sender).Tag);
  MsgResult := MsgBtnResult[B];
  if B = mbHelp then
    Application.HelpContext(HelpContext)
end;

procedure TAssistantBalloon.ShowTipsDlg;
begin
  if Visible then Hide;
  Origin := Point(0,0);
  BalloonKind := bkTips;
  AutoHideDelay := 0;
  BalloonShape := bsSpeakShape;
  CreateTipsButtons;
  HelpContext := 0;
  Rebuild(True);
  Show;
end;

procedure TAssistantBalloon.RebuildTipsDlg;
var
  R: TRect;
  B: TTipsDlgBtn;
  BtnsWidth: Integer;
  TextHeight: Integer;
  ButtonLeft, ButtonTop: integer;
begin
  SelectTip;
  TextHeight := Canvas.TextHeight('H');
  BtnsWidth := TipsBtnCount * (MsgBtnWidth + ControlSpace) - ControlSpace;
  SetRect(R, 0, 0, MaxTipsDlgWidth - 2 * ControlSpace, 0);
  if R.Right < BtnsWidth then R.Right := BtnsWidth;
  DrawText(Canvas.Handle, PChar(MsgText.Caption), Length(MsgText.Caption), R,
    DrawTextBiDiModeFlags(DT_NOPREFIX or DT_WORDBREAK or DT_CALCRECT));
  if R.Right < (MinTipsDlgWidth - 2 * ControlSpace) then
    R.Right := MinTipsDlgWidth - 2 * ControlSpace;
  if R.Right < BtnsWidth then R.Right := BtnsWidth;
  if R.Bottom < (2 * TextHeight) then
    Inc(R.Bottom, TextHeight);
  if TipIcon.Height >= TextHeight then
    TipIcon.Top := Origin.Y + ControlSpace
  else
    TipIcon.Top := Origin.Y + ControlSpace + (TextHeight - TipIcon.Height) div 2;
  if TipIcon.Height >= TextHeight then
    OffsetRect(R, Origin.X + ControlSpace, TipIcon.Top +
      TipIcon.Height - TextHeight + 1)
  else
    OffsetRect(R, Origin.X + ControlSpace, TipIcon.Top -
      (TextHeight - TipIcon.Height) div 2);
  MsgText.BoundsRect := R;
  ButtonTop := R.Bottom + ControlSpace;
  if UseRightToLeftAlignment then
  begin
    TipIcon.Left := R.Right - TipIcon.Width - ControlSpace;
    ButtonLeft := R.Left;
    for B := Low(TTipsDlgBtn) to High(TTipsDlgBtn) do
      if Assigned(TipsBtn[B]) then
      begin
        TipsBtn[B].Left := ButtonLeft;
        TipsBtn[B].Top := ButtonTop;
        Inc(ButtonLeft, TipsBtnWidth + ControlSpace);
      end;
  end
  else
  begin
    TipIcon.Left := Origin.X + 2 * ControlSpace;
    ButtonLeft := R.Right - TipsBtnWidth;
    for B := Low(TTipsDlgBtn) to High(TTipsDlgBtn) do
      if Assigned(TipsBtn[B]) then
      begin
        TipsBtn[B].Left := ButtonLeft;
        TipsBtn[B].Top := ButtonTop;
        Dec(ButtonLeft, TipsBtnWidth + ControlSpace);
      end;
  end;
  BoxWidth := MsgText.Width + 2 * ControlSpace;
  if TipsBtnCount = 0 then
    BoxHeight := ButtonTop + ControlSpace - Origin.Y
  else
    BoxHeight := ButtonTop + BtnHeight + ControlSpace - Origin.Y;
end;

procedure TAssistantBalloon.SelectTip;
begin
  with TipsDlgOptions do
  begin
    if (TipIndex >= 0) and (TipIndex < TipsDlgOptions.Tips.Count) then
      MsgText.Caption := TipPadStr + Tips[TipIndex]
    else
      MsgText.Caption := EmptyStr;
    if Assigned(TipsBtn[tbNext]) then
      TipsBtn[tbNext].Enabled := TipIndex < Tips.Count-1;
    if Assigned(TipsBtn[tbBack]) then
      TipsBtn[tbBack].Enabled := TipIndex > 0;
    if Assigned(OnTipsTipChange) then
      OnTipsTipChange(Self);
  end;
end;

procedure TAssistantBalloon.CreateTipsButtons;
var
  ThisBtnWidth: Integer;
  B: TTipsDlgBtn;
begin
  TipsBtnCount := 0;
  TipsBtnWidth := MinButtonWidth;
  for B := Low(TTipsDlgBtn) to High(TTipsDlgBtn) do
  begin
    if B in TipsDlgOptions.VisibleButtons then
    begin
      Inc(TipsBtnCount);
      ThisBtnWidth := Canvas.TextWidth(DefTipsDlgBtns[B]) + ButtonSpace;
      if TipsBtnWidth < ThisBtnWidth then TipsBtnWidth := ThisBtnWidth;
    end;
  end;
  for B := Low(TTipsDlgBtn) to High(TTipsDlgBtn) do
  begin
    if B in TipsDlgOptions.VisibleButtons then
    begin
      If not Assigned(TipsBtn[B]) then
      begin
        TipsBtn[B]:= TOffice97Button.Create(Self);
        TipsBtn[B].Parent := Self;
        TipsBtn[B].Tag := Ord(B);
        TipsBtn[B].WordWrap := False;
        TipsBtn[B].OnClick := TipsBtnClick;
      end;
      TipsBtn[B].Color := Options.Color;
      TipsBtn[B].Caption := DefTipsDlgBtns[B];
      TipsBtn[B].Width := TipsBtnWidth;
      TipsBtn[B].Height := BtnHeight;
      TipsBtn[B].Visible := (BalloonKind = bkTips);
    end
    else if Assigned(TipsBtn[B]) then
    begin
      TipsBtn[B].Free;
      TipsBtn[B] := nil;
    end;
  end;
end;

procedure TAssistantBalloon.CreateTipsPadStr;
var
  N: Integer;
begin
  TipPadStr := EmptyStr;
  for N := 1 to (2 * ControlSpace + TipIcon.Width) div Canvas.TextWidth(' ') do
    TipPadStr := TipPadStr + ' ';
end;

procedure TAssistantBalloon.TipsBtnClick(Sender: TObject);
begin
  case TTipsDlgBtn(TOffice97Button(Sender).Tag) of
    tbClose: Close;
    tbNext: TipsDlgOptions.TipIndex := TipsDlgOptions.TipIndex + 1;
    tbBack: TipsDlgOptions.TipIndex := TipsDlgOptions.TipIndex - 1;
  end;
end;

procedure TAssistantBalloon.ShowGuideDlg;
begin
  if Visible then Hide;
  Origin := Point(0,0);
  BalloonKind := bkGuide;
  AutoHideDelay := 0;
  BalloonShape := bsSpeakShape;
  CreateGuideButtons;
  HelpContext := 0;
  TopTopic := 0;
  Rebuild(True);
  if BoxHeight > 2 * ControlSpace then
    Show;
end;

procedure TAssistantBalloon.RebuildGuideDlg;
var
  R: TRect;
  B: TGuideDlgBtn;
  BtnsWidth, I: Integer;
  ButtonLeft, ButtonTop: integer;
  OptionsBtn, SearchBtn: Boolean;
begin
  Title.Caption := GuideDlgOptions.Title;
  MsgText.Caption := GuideDlgOptions.Description;
  UserRequest.OnChange := nil;
  UserRequest.Text := GuideDlgOptions.Request;
  UserRequest.OnChange := UserRequestChange;
  BtnsWidth := GuideBtnCount * (GuideBtnWidth + ControlSpace) - ControlSpace;
  if (MinGuideDlgWidth - 2 * ControlSpace) > BtnsWidth then
    TopicsPanel.Width := MinGuideDlgWidth - 2 * ControlSpace
  else
    TopicsPanel.Width := BtnsWidth;
  if Assigned(GuideBtn[gbSearch]) and (TopicsPanel.Width < GuideBtn[gbSearch].Width) then
    TopicsPanel.Width := GuideBtn[gbSearch].Width;
  if TopicsPanel.Width < Title.Width then
    TopicsPanel.Width := Title.Width;
  TopicsPanel.Left := Origin.X + ControlSpace;
  MsgText.Left := TopicsPanel.Left;
  UserRequest.Left := TopicsPanel.Left;
  DividerTop.Left := TopicsPanel.Left;
  DividerBottom.Left := TopicsPanel.Left;
  Title.Left := TopicsPanel.Left;
  MsgText.Width := TopicsPanel.Width;
  UserRequest.Width := TopicsPanel.Width;
  DividerTop.Width := TopicsPanel.Width;
  DividerBottom.Width := TopicsPanel.Width;
  Title.Width := TopicsPanel.Width;
  Title.Top := Origin.Y + ControlSpace;
  SetRect(R, 0, 0, TopicsPanel.Width, 0);
  DrawText(Canvas.Handle, PChar(MsgText.Caption), Length(MsgText.Caption),
    R, DrawTextBiDiModeFlags(DT_NOPREFIX or DT_WORDBREAK or DT_CALCRECT));
  MsgText.Height := R.Bottom;
  if Title.Caption = EmptyStr then
    MsgText.Top := Title.Top
  else
    MsgText.Top := Title.Top + Title.Height + 2 * ControlSpace;
  if R.Bottom = 0 then
    TopicsPanel.Top := MsgText.Top
  else
    TopicsPanel.Top := MsgText.Top + MsgText.Height + ControlSpace;
  ButtonTop := 0;
  GuideBtn[gbBackPage].Visible := TopTopic > 0;
  if GuideBtn[gbBackPage].Visible then
  begin
    GuideBtn[gbBackPage].TabOrder := 0;
    GuideBtn[gbBackPage].Top := ButtonTop;
    GuideBtn[gbBackPage].Width := TopicsPanel.Width;
    Inc(ButtonTop, GuideBtn[gbBackPage].Height + ControlSpace);
  end;
  VisibleTopics := 0;
  for I := TopTopic to GuideDlgOptions.Topics.Count-1 do
  begin
    if VisibleTopics >= TopicBtns.Count then
    begin
      TopicBtns.Add(TOffice97Button.Create(TopicsPanel));
      with TOffice97Button(TopicBtns[VisibleTopics]) do
      begin
        GuideBtn[gbBackPage].TabOrder := VisibleTopics + 1;
        Parent := TopicsPanel;
        WordWrap := True;
        AutoHeight := True;
        Left := 0;
        ControlType := bsRadioButton;
        OnClick := GuideBtnClick;
      end;
    end;
    with TOffice97Button(TopicBtns[VisibleTopics]) do
    begin
      Top := ButtonTop;
      Width := TopicsPanel.Width;
      Color := Options.Color;
      Caption := GuideDlgOptions.Topics[I];
      Tag := TopTopic + VisibleTopics;
      if ((Top + Height + ControlSpace) <= MaxTopicHeight) or (VisibleTopics = 0) then
      begin
        Inc(ButtonTop, Height + ControlSpace);
        Visible := True;
        Inc(VisibleTopics);
      end
      else
        Break;
    end;
  end;
  GuideBtn[gbNextPage].Visible := (TopTopic + VisibleTopics) < GuideDlgOptions.Topics.Count;
  if GuideBtn[gbNextPage].Visible then
  begin
    GuideBtn[gbNextPage].TabOrder := VisibleTopics + 1;
    GuideBtn[gbNextPage].Width := TopicsPanel.Width;
    Inc(ButtonTop, GuideBtn[gbNextPage].Height + ControlSpace);
  end;
  if (TopTopic = 0) or (VisibleTopics = 0) then
    TopicsPanel.Height := ButtonTop
  else if ButtonTop > TopicsPanel.Height then
    if VisibleTopics > 1 then
      Dec(VisibleTopics)
    else
      TopicsPanel.Height := ButtonTop;
  if GuideBtn[gbNextPage].Visible then
    GuideBtn[gbNextPage].Top := TopicsPanel.Height - (GuideBtn[gbNextPage].Height + ControlSpace);
  for I := VisibleTopics to TopicBtns.Count-1 do
    TOffice97Button(TopicBtns[I]).Visible := False;
  DividerTop.Visible := (VisibleTopics > 0) or (Title.Caption <> EmptyStr) or (MsgText.Caption <> EmptyStr);
  DividerTop.Top := TopicsPanel.Top + TopicsPanel.Height;
  UserRequest.Top := DividerTop.Top + ControlSpace;
  if Assigned(GuideBtn[gbSearch]) and GuideDlgOptions.RequestBox then
  begin
    GuideBtn[gbSearch].Top := UserRequest.Top + UserRequest.Height + ControlSpace - 2;
    GuideBtn[gbSearch].Left := Origin.X + ControlSpace + (UserRequest.Width - GuideBtn[gbSearch].Width) div 2;
    DividerBottom.Top := GuideBtn[gbSearch].Top + BtnHeight + ControlSpace;
  end
  else if GuideDlgOptions.RequestBox then
    DividerBottom.Top := UserRequest.Top + UserRequest.Height + ControlSpace
  else
    DividerBottom.Top := DividerTop.Top;
  ButtonTop := DividerBottom.Top + ControlSpace;
  ButtonLeft := Origin.X + ControlSpace + (UserRequest.Width - BtnsWidth) div 2;
  if UseRightToLeftAlignment then
  begin
    for B := Low(TGuideDlgCmdBtn) to Pred(High(TGuideDlgCmdBtn)) do
      if Assigned(GuideBtn[B]) then
      begin
        GuideBtn[B].Left := ButtonLeft;
        GuideBtn[B].Top := ButtonTop;
        Inc(ButtonLeft, GuideBtnWidth + ControlSpace);
      end
  end
  else
  begin
    for B := Pred(High(TGuideDlgCmdBtn)) downto Low(TGuideDlgCmdBtn) do
      if Assigned(GuideBtn[B]) then
      begin
        GuideBtn[B].Left := ButtonLeft;
        GuideBtn[B].Top := ButtonTop;
        Inc(ButtonLeft, GuideBtnWidth + ControlSpace);
      end
  end;
  DividerTop.Visible := DividerTop.Visible and GuideDlgOptions.RequestBox;
  UserRequest.Visible := GuideDlgOptions.RequestBox;
  if Assigned(GuideBtn[gbSearch]) then
    GuideBtn[gbSearch].Visible := GuideDlgOptions.RequestBox;
  DividerBottom.Visible := GuideBtnCount > 0;
  BoxWidth := UserRequest.Width + 2 * ControlSpace;
  if GuideBtnCount = 0 then
    BoxHeight := DividerBottom.Top + ControlSpace div 2 - Origin.Y
  else
    BoxHeight := ButtonTop + BtnHeight + ControlSpace - Origin.Y;
  OptionsBtn := Assigned(GuideDlgOptions.OnGuideOptionsClick);
  SearchBtn := Assigned(GuideDlgOptions.OnGuideSearchClick);
  if Assigned(GuideDlgOptions.OnGuideGetBtnStates) then
    GuideDlgOptions.OnGuideGetBtnStates(Self, OptionsBtn, SearchBtn);
  if Assigned(GuideBtn[gbOptions]) then
    GuideBtn[gbOptions].Enabled := OptionsBtn;
  if Assigned(GuideBtn[gbTips]) then
    GuideBtn[gbTips].Enabled := TipsDlgOptions.Tips.Count > 0;
  if Assigned(GuideBtn[gbSearch]) then
    GuideBtn[gbSearch].Enabled := SearchBtn;
  UserRequest.SelectAll;
  if UserRequest.Visible then
    ActiveControl := UserRequest;
end;

procedure TAssistantBalloon.CreateGuideButtons;
var
  ThisBtnWidth: Integer;
  B: TGuideDlgBtn;
begin
  GuideBtnCount := 0;
  GuideBtnWidth := MinButtonWidth;
  for B := Low(TGuideDlgCmdBtn) to High(TGuideDlgCmdBtn) do
  begin
    if (B in GuideDlgOptions.VisibleButtons) and (B <> gbSearch) then
    begin
      Inc(GuideBtnCount);
      ThisBtnWidth := Canvas.TextWidth(DefGuideDlgBtns[B]) + ButtonSpace;
      if GuideBtnWidth < ThisBtnWidth then GuideBtnWidth := ThisBtnWidth;
    end;
  end;
  for B := Low(TGuideDlgCmdBtn) to High(TGuideDlgCmdBtn) do
  begin
    if B in GuideDlgOptions.VisibleButtons then
    begin
      If not Assigned(GuideBtn[B]) then
      begin
        GuideBtn[B]:= TOffice97Button.Create(Self);
        GuideBtn[B].Parent := Self;
        GuideBtn[B].Tag := Ord(B);
        GuideBtn[B].WordWrap := False;
        GuideBtn[B].OnClick := GuideBtnClick;
      end;
      GuideBtn[B].Color := Options.Color;
      GuideBtn[B].Caption := DefGuideDlgBtns[B];
      GuideBtn[B].Height := BtnHeight;
      if B = gbSearch then
      begin
        GuideBtn[B].Width := Canvas.TextWidth(DefGuideDlgBtns[B]) + 2 * ButtonSpace;
        GuideBtn[B].ParentFont := True;
        GuideBtn[B].Font.Style := GuideBtn[B].Font.Style + [fsBold];
      end
      else
        GuideBtn[B].Width := GuideBtnWidth;
      GuideBtn[B].Visible := (BalloonKind = bkGuide);
    end
    else if Assigned(GuideBtn[B]) then
    begin
      GuideBtn[B].Free;
      GuideBtn[B] := nil;
    end;
  end;
  for B := Low(TGuideDlgNavBtn) to High(TGuideDlgNavBtn) do
  begin
    If not Assigned(GuideBtn[B]) then
    begin
      GuideBtn[B]:= TOffice97Button.Create(TopicsPanel);
      GuideBtn[B].Parent := TopicsPanel;
      GuideBtn[B].Tag := Ord(B);
      GuideBtn[B].WordWrap := False;
      GuideBtn[B].AutoHeight := True;
      GuideBtn[B].Left := 0;
      if B = gbBackPage then
        GuideBtn[B].ControlType := bsUpButton
      else
        GuideBtn[B].ControlType := bsDownButton;
      GuideBtn[B].OnClick := GuideBtnClick;
    end;
    GuideBtn[B].Color := Options.Color;
    GuideBtn[B].Caption := DefGuideDlgBtns[B];
    GuideBtn[B].Visible := False;
  end;
end;

procedure TAssistantBalloon.DoSearch;
begin
  if Assigned(GuideDlgOptions.OnGuideSearchClick) and (UserRequest.Text <> EmptyStr) then
  begin
    Hide;
    TopTopic := 0;
    GuideDlgOptions.Topics.BeginUpdate;
    try
      GuideDlgOptions.OnGuideSearchClick(Self);
    finally
      GuideDlgOptions.Topics.EndUpdate;
      RebuildGuideDlg;
    end;
    Show;
  end;
end;

procedure TAssistantBalloon.ShowNextPage;
var
  H: Integer;
begin
  if (TopTopic + VisibleTopics) < GuideDlgOptions.Topics.Count then
  begin
    Inc(TopTopic, VisibleTopics);
    H := TopicsPanel.Height;
    RebuildGuideDlg;
    if H <> TopicsPanel.Height then Prepare;
    if (TopTopic + VisibleTopics) = GuideDlgOptions.Topics.Count then
    ActiveControl := GuideBtn[gbBackPage];
  end;
end;

procedure TAssistantBalloon.ShowBackPage;
var
  I, H, T: Integer;
  B: TOffice97Button;
begin
  if TopTopic > 0 then
  begin
    B := TOffice97Button(TopicBtns[VisibleTopics-1]);
    H := TopicsPanel.Height - (GuideBtn[gbNextPage].Height + ControlSpace);
    T := TopTopic;
    for I := TopTopic-1 downto 0 do
    begin
      B.Caption := GuideDlgOptions.Topics[I];
      if (T = TopTopic) or (H >= B.Height + ControlSpace) then
      begin
        Dec(T);
        Dec(H, B.Height + ControlSpace);
      end
      else
        Break;
    end;
    if (T > 0) and (H < GuideBtn[gbNextPage].Height + ControlSpace) then
      Inc(T);
    TopTopic := T;
    H := TopicsPanel.Height;
    RebuildGuideDlg;
    if H <> TopicsPanel.Height then Prepare;
    if TopTopic = 0 then
    ActiveControl := GuideBtn[gbNextPage];
  end;
end;

procedure TAssistantBalloon.GuideBtnClick(Sender: TObject);
begin
  if Sender is TOffice97Button then
  begin
    if TOffice97Button(Sender).ControlType = bsRadioButton then
    begin
      if Assigned(GuideDlgOptions.OnGuideTopicClick) then
        GuideDlgOptions.OnGuideTopicClick(Self, TOffice97Button(Sender).Tag);
    end
    else
      case TGuideDlgBtn(TOffice97Button(Sender).Tag) of
        gbClose:
          Close;
        gbOptions:
          if Assigned(GuideDlgOptions.OnGuideOptionsClick) then
            GuideDlgOptions.OnGuideOptionsClick(Self);
        gbTips:
          if TipsDlgOptions.Tips.Count > 0 then
            ShowTipsDlg;
        gbBackPage:
          ShowBackPage;
        gbNextPage:
          ShowNextPage;
        gbSearch:
          DoSearch;
      end;
  end;
end;

procedure TAssistantBalloon.UserRequestChange(Sender: TObject);
begin
  GuideDlgOptions.InternalChange := True;
  try
    GuideDlgOptions.Request := UserRequest.Text;
  finally
    GuideDlgOptions.InternalChange := False;
  end;
  if Assigned(GuideDlgOptions.OnGuideRequestChange) then
    GuideDlgOptions.OnGuideRequestChange(Self);
end;

procedure TAssistantBalloon.UserRequestKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = Chr(VK_RETURN) then DoSearch;
end;

end.
