unit oaPanel;

{ Office 97 Assistant Package v1.0 }
{ Copyright  1999 Kambiz R. Khojasteh }
{ email: Khojasteh@www.dci.co.ir }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, oaDirect, oaTypes;

type

  TAssistantPanel = class(TCustomPanel)
  private
    FAbout: TAboutMe;
    Director: TAssistantDirector;
    FAutoSize: Boolean;
    FStretch: Boolean;
    FBgPicture: TPicture;
    FBgMode: TBgMode;
    Frame: TBitmap;
    FOnPlayStart: TPlayEvent;
    FOnPlayStop: TPlayEvent;
    FOnIdle: TIdleEvent;
    FOnChange: TNotifyEvent;
    procedure SetAutoSize(Value: Boolean);
    procedure SetStretch(Value: Boolean);
    procedure SetMute(Value: Boolean);
    function GetMute: Boolean;
    function GetSpeed: Word;
    procedure SetSpeed(Value: Word);
    function GetPriority: TThreadPriority;
    procedure SetPriority(Value: TThreadPriority);
    procedure SetAutoIdle(Value: Boolean);
    function GetAutoIdle: Boolean;
    procedure SetActorLibrary(Value: String);
    function GetActorLibrary: String;
    function GetActorName: String;
    function GetActorVersion: String;
    function GetActorCopyright: String;
    function GetAnimationCount: Word;
    function GetAnimations(Index: Word): TAnimation;
    procedure SetBgPicture(Value: TPicture);
    procedure SetBgMode(Value: TBgMode);
    function GetPlaying: Boolean;
    function GetEmpty: Boolean;
    procedure ActorChanged(Sender: TObject);
    procedure BgPictureChanged(Sender: TObject);
    procedure FrameChanged(Sender: TObject; FrameBitmap: TBitmap);
    procedure PlayStarted(Sendr: TObject; RequestID: TRequestID);
    procedure PlayStopped(Sendr: TObject; RequestID: TRequestID);
    procedure IdleEntered(Sendr: TObject; Level: Integer; var Handled: Boolean);
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AdjustClientSize;
    function Play(Index: Word): TRequestID; overload;
    function Play(Name: String): TRequestID; overload;
    function Play(Action: TActorAction): TRequestID; overload;
    function WaitFor(RequestID: TRequestID): Boolean;
    procedure Stop(RequestID: TRequestID);
    procedure StopAll;
    function IsValid(RequestID: TRequestID): Boolean;
    function GestureAt(X, Y: Integer): TRequestID;
    function GlanceAt(X, Y: Integer): TRequestID;
    function LookAt(X, Y: Integer): TRequestID;
    property Empty: Boolean read GetEmpty;
    property Playing: Boolean read GetPlaying;
    property ActorName: String read GetActorName;
    property ActorVersion: String read GetActorVersion;
    property ActorCopyright: String read GetActorCopyright;
    property AnimationCount: Word read GetAnimationCount;
    property Animations[Index: Word]: TAnimation read GetAnimations;
  published
    property About: TAboutMe read FAbout write FAbout stored False;
    property ActorLibrary: String read GetActorLibrary write SetActorLibrary;
    property Align;
    property Anchors;
    property AutoIdle: Boolean read GetAutoIdle write SetAutoIdle default True;
    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BgMode: TBgMode read FBgMode write SetBgMode default bmTiled;
    property BgPicture: TPicture read FBgPicture write SetBgPicture;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Height default 130;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property Priority: TThreadPriority read GetPriority write SetPriority default tpNormal;
    property Mute: Boolean read GetMute write SetMute default False;
    property ShowHint;
    property Speed: Word read GetSpeed write SetSpeed default 100;
    property Stretch: Boolean read FStretch write SetStretch default False;
    property Visible;
    property Width default 130;
    property OnCanResize;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnConstrainedResize;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnIdle: TIdleEvent read FOnIdle write FOnIdle;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnPlayStart: TPlayEvent read FOnPlayStart write FOnPlayStart;
    property OnPlayStop: TPlayEvent read FOnPlayStop write FOnPlayStop;
    property OnResize;
    property OnStartDrag;
  end;

implementation

uses
  oaEngine;

procedure DrawBevel(Canvas: TCanvas; Rect: TRect; Color: TColor;
  BevelInner, BevelOuter: TPanelBevel; BorderWidth, BevelWidth: Integer);
var
  TopColor, BottomColor: TColor;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  if BevelOuter <> bvNone then
  begin
    AdjustColors(BevelOuter);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
  Frame3D(Canvas, Rect, Color, Color, BorderWidth);
  if BevelInner <> bvNone then
  begin
    AdjustColors(BevelInner);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
end;

constructor TAssistantPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Director := TAssistantDirector.Create;
  Director.Animator.OnPaintFrame := FrameChanged;
  Director.OnActorChange := ActorChanged;
  Director.OnPlayStart := PlayStarted;
  Director.OnPlayStop := PlayStopped;
  Director.OnIdle := IdleEntered;
  Director.AutoIdle := True;
  FBgPicture := TPicture.Create;
  FBgPicture.OnChange := BgPictureChanged;
  FBgMode := bmTiled;
  FStretch := False;
  FAutoSize := True;
  Frame := TBitmap.Create;
  DoubleBuffered := True;
  Width := 130;
  Height := 130;
  Locked := True;
  TabStop := False;
end;

destructor TAssistantPanel.Destroy;
begin
  Director.Free;
  BGPicture.Free;
  Frame.Free;
  inherited Destroy;
end;

procedure TAssistantPanel.SetAutoSize(Value: Boolean);
begin
  if AutoSize <> Value then
  begin
    FAutoSize := Value;
    if AutoSize then AdjustClientSize;
  end;
end;

procedure TAssistantPanel.SetStretch(Value: Boolean);
begin
  if Stretch <> Value then
  begin
    FStretch := Value;
    Invalidate;
  end;
end;

procedure TAssistantPanel.SetMute(Value: Boolean);
begin
  Director.Animator.Mute := Value;
end;

function TAssistantPanel.GetMute: Boolean;
begin
  Result := Director.Animator.Mute;
end;

procedure TAssistantPanel.SetSpeed(Value: Word);
begin
  if Value <> 0 then Director.Animator.Speed := Value;
end;

function TAssistantPanel.GetSpeed: Word;
begin
  Result := Director.Animator.Speed;
end;

procedure TAssistantPanel.SetPriority(Value: TThreadPriority);
begin
  Director.Animator.Priority := Value;
end;

function TAssistantPanel.GetPriority: TThreadPriority;
begin
  Result := Director.Animator.Priority;
end;

procedure TAssistantPanel.SetAutoIdle(Value: Boolean);
begin
  Director.AutoIdle := Value;
end;

function TAssistantPanel.GetAutoIdle: Boolean;
begin
  Result := Director.AutoIdle;
end;

procedure TAssistantPanel.SetActorLibrary(Value: String);
begin
  try
    Director.Actor.LibraryName := Value;
  except
    if not (csLoading in ComponentState) then raise;
  end;
end;

function TAssistantPanel.GetActorLibrary: String;
begin
  Result := Director.Actor.LibraryName;
end;

function TAssistantPanel.GetActorName: String;
begin
  Result := Director.Actor.ActorName;
end;

function TAssistantPanel.GetActorVersion: String;
begin
  Result := Director.Actor.Version;
end;

function TAssistantPanel.GetActorCopyright: String;
begin
  Result := Director.Actor.Copyright;
end;

function TAssistantPanel.GetAnimationCount: Word;
begin
  Result := Director.Actor.AnimationCount;
end;

function TAssistantPanel.GetAnimations(Index: Word): TAnimation;
begin
  Result := Director.Actor.Animation[Index];
end;

procedure TAssistantPanel.SetBgPicture(Value: TPicture);
begin
  FBgPicture.Assign(Value);
end;

procedure TAssistantPanel.SetBgMode(Value: TBgMode);
begin
  if BgMode <> Value then
  begin
    FBgMode := Value;
    if FBgPicture.Graphic <> nil then
      Invalidate;
  end;
end;

procedure TAssistantPanel.ActorChanged(Sender: TObject);
begin
  if not (csLoading in ComponentState) then
  begin
    if not Director.Actor.Active then
    begin
      Frame.ReleaseHandle;
      if Visible then Invalidate;
    end
    else
    begin
      if AutoSize then AdjustClientSize;
      Play(aaRestPose);
    end;
    if Assigned(FOnChange) then
      FOnChange(Self);
  end;
end;

procedure TAssistantPanel.BgPictureChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TAssistantPanel.FrameChanged(Sender: TObject; FrameBitmap: TBitmap);
begin
  if not (csDestroying in ComponentState) then
  begin
    Frame.Assign(FrameBitmap);
    Refresh;
  end;
end;

procedure TAssistantPanel.PlayStarted(Sendr: TObject; RequestID: TRequestID);
begin
  if Assigned(FOnPlayStart) and not (csDesigning in ComponentState) then
    FOnPlayStart(Self, RequestID);
end;

procedure TAssistantPanel.PlayStopped(Sendr: TObject; RequestID: TRequestID);
begin
  if Assigned(FOnPlayStop) and not (csDesigning in ComponentState) and
    not (csDestroying in ComponentState) then
    FOnPlayStop(Self, RequestID);
end;

procedure TAssistantPanel.IdleEntered(Sendr: TObject; Level: Integer;
  var Handled: Boolean);
begin
  if not Visible or (csDesigning in ComponentState) then
    Handled := True
  else if Assigned(FOnIdle) then
    FOnIdle(Self, Level, Handled);
end;

procedure TAssistantPanel.AdjustClientSize;
begin
  if Director.Actor.Active then
    with Director.Actor.AnimationSize do
      SetBounds(Left, Top, cX, cY);
end;

procedure TAssistantPanel.WMEraseBkgnd(var Message: TMessage);
begin
  Message.Result := 1;
end;

procedure TAssistantPanel.Loaded;
begin
  inherited Loaded;
  if Director.Actor.Active then
  begin
    if AutoSize then AdjustClientSize;
    Play(aaRestPose);
  end;
end;

procedure TAssistantPanel.Paint;
var
  R, iR: TRect;
begin
  R := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.FillRect(R);
  if (BevelOuter <> bvNone) or (BevelInner <> bvNone) then
  begin
    if (BevelOuter = bvNone) or (BevelInner = bvNone) then
      InflateRect(R, -BevelWidth, -BevelWidth)
    else
      InflateRect(R, -2 * BevelWidth, -2 * BevelWidth);
  end;
  if BgPicture.Graphic <> nil then
  begin
    case FBgMode of
      bmTiled: DrawTiled(Canvas, R, BgPicture.Graphic);
      bmCentered: Canvas.Draw((R.Right - R.Left - BgPicture.Width) div 2,
        (R.Bottom - R.Top - BgPicture.Height) div 2, BgPicture.Graphic);
      bmStretched: Canvas.StretchDraw(R, BgPicture.Graphic);
    end;
  end;
  if not Frame.Empty then
  begin
    iR := Rect(0, 0, Frame.Width, Frame.Height);
    Canvas.StretchDraw(ScaleImageToRect(iR, R, Stretch), Frame);
  end;
  DrawBevel(Canvas, ClientRect, Color, BevelInner, BevelOuter,
      BorderWidth, BevelWidth);
end;

function TAssistantPanel.Play(Index: Word): TRequestID;
begin
  Result := Director.Play(Index);
end;

function TAssistantPanel.Play(Name: String): TRequestID;
begin
  Result := Director.Play(Name);
end;

function TAssistantPanel.Play(Action: TActorAction): TRequestID;
begin
  Result := Director.Play(Action);
end;

function TAssistantPanel.WaitFor(RequestID: TRequestID): Boolean;
begin
  Result := Director.WaitFor(RequestID);
end;

procedure TAssistantPanel.Stop(RequestID: TRequestID);
begin
  Director.Stop(RequestID);
end;

procedure TAssistantPanel.StopAll;
begin
  Director.StopAll;
end;

function TAssistantPanel.GetPlaying: Boolean;
begin
  Result := Director.Playing;
end;

function TAssistantPanel.GetEmpty: Boolean;
begin
  Result := not Director.Actor.Active;
end;

function TAssistantPanel.IsValid(RequestID: TRequestID): Boolean;
begin
  Result := Director.IsValid(RequestID);
end;

function TAssistantPanel.GestureAt(X, Y: Integer): TRequestID;
var
  Action: TActorAction;
  RefP: TPoint;
begin
  RefP := ClientToScreen(Point(Width div 2, Height div 2));
  Action := Director.OrientationOf(oaGesture, RefP, Point(X, Y));
  Result := Play(Action);
end;

function TAssistantPanel.GlanceAt(X, Y: Integer): TRequestID;
var
  Action: TActorAction;
  RefP: TPoint;
begin
  RefP := ClientToScreen(Point(Width div 2, Height div 2));
  Action := Director.OrientationOf(oaGlance, RefP, Point(X, Y));
  Result := Play(Action);
end;

function TAssistantPanel.LookAt(X, Y: Integer): TRequestID;
var
  Action: TActorAction;
  RefP: TPoint;
begin
  RefP := ClientToScreen(Point(Width div 2, Height div 2));
  Action := Director.OrientationOf(oaLook, RefP, Point(X, Y));
  Result := Play(Action);
end;

end.
