unit oaProps;

{ Office 97 Assistant Package v1.0 }
{ Copyright  1999 Kambiz R. Khojasteh }
{ email: Khojasteh@www.dci.co.ir }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, StdCtrls, ExtCtrls, Grids, oaTypes, oaPanel;

type

{ TAboutMeProperty }

  TAboutMeProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TActorLibPropertyEditor }

  TActorLibPropertyEditor = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TActorLibComponentEditor }

  TActorLibComponentEditor = class(TComponentEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TActorLibDialog }

  TActorLibDialog = class(TForm)
    OpenDialog: TOpenDialog;
    BtnOK: TButton;
    BtnCancel: TButton;
    Animations: TStringGrid;
    Panel: TPanel;
    BtnLoad: TButton;
    BtnClear: TButton;
    Information: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ActorLibrary: TLabel;
    Label3: TLabel;
    ActorName: TLabel;
    ActorCopyright: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure AnimationsKeyPress(Sender: TObject; var Key: Char);
    procedure AnimationsDblClick(Sender: TObject);
    procedure BtnLoadClick(Sender: TObject);
    procedure BtnClearClick(Sender: TObject);
  private
    Preview: TAssistantPanel;
    function GetLibraryName: String;
    procedure SetLibraryName(Value: String);
  public
    function Execute: Boolean;
  published
    property LibraryName: String read GetLibraryName write SetLibraryName;
  end;

function SelectActor(Caption: String; var Filename: String): Boolean;

implementation

{$R *.DFM}

uses
  oaAssist, oaAbout;

const
  BooleanNames: array[Boolean] of string = ('No', 'Yes');
  ExitTypeNames: array[TExitType] of String =
    ('None', 'Return Animation', 'Use Exit Branching');
  ActorActionNames: array[TActorAction] of String =
    ('None', 'Acknowledge', 'Alert', 'Announce', 'Blink', 'Confused',
     'Congratulate', 'Decline', 'DoMagic', 'DontRecognize', 'Explain',
     'GestureDown', 'GestureLeft', 'GestureRight', 'GestureUp', 'GetAttention',
     'GetAttentionContinued', 'GlanceDown', 'GlanceLeft', 'GlanceRight',
     'GlanceUp', 'Greet', 'Hear', 'Hearing', 'Hide', 'IdlingLevel1',
     'IdlingLevel2', 'IdlingLevel3', 'Listen', 'Listening', 'LookDown',
     'LookLeft', 'LookRight', 'LookUp', 'MoveDown', 'MoveLeft', 'MoveRight',
     'MoveUp', 'Pleased', 'Process', 'Processing', 'Read', 'ReadContinued',
     'Reading', 'RestPose', 'Sad', 'Search', 'Searching', 'Show', 'Speak',
     'Speaking', 'StartListening', 'StopListening', 'Suggest', 'Surprised',
     'Think', 'Thinking', 'Uncertain', 'Wave', 'Write', 'WriteContinued',
     'Writing');

{ TAboutMeProperty }

function TAboutMeProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TAboutMeProperty.GetValue: string;
begin
  Result := '(About)'
end;

procedure TAboutMeProperty.Edit;
begin
  TAssistantAbout.Execute(GetComponent(0).ClassName + ' v1.0');
end;

{ TActorLibPropertyEditor }

function TActorLibPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

procedure TActorLibPropertyEditor.Edit;
const
  Editing: Boolean = False;
var
  Filename: String;
begin
  if not Editing then
  begin
    Editing := True;
    try
     Filename := GetStrValue;
     if SelectActor(GetComponent(0).GetNamePath + '.ActorLibrary', Filename) then
       SetStrValue(Filename);
    finally
      Editing := False;
    end;
  end;
end;

{ TActorLibComponentEditor }

procedure TActorLibComponentEditor.ExecuteVerb(Index: Integer);
const
  Editing: Boolean = False;
var
  Filename: String;
begin
  if (Index = 0) and not Editing then
  begin
    Editing := True;
    try
     if Component is TAssistantPanel then
       Filename := TAssistantPanel(Component).ActorLibrary
     else if Component is TAssistant then
       Filename := TAssistant(Component).ActorLibrary
     else Exit;
     if SelectActor(Component.GetNamePath + '.ActorLibrary', Filename) then
     begin
       if Component is TAssistantPanel then
         TAssistantPanel(Component).ActorLibrary := Filename
       else if Component is TAssistant then
         TAssistant(Component).ActorLibrary := Filename;
       Designer.Modified;
     end;
    finally
      Editing := False;
    end;
  end;
end;

function TActorLibComponentEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then
    Result := 'Select Actor...'
  else
    Result := '';
end;

function TActorLibComponentEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

{ TActorLibDialog }

function TActorLibDialog.Execute: Boolean;
begin
  Result := (ShowModal = mrOK);
end;

function TActorLibDialog.GetLibraryName: String;
begin
  Result := Preview.ActorLibrary;
end;

procedure TActorLibDialog.SetLibraryName(Value: String);
const
  Welcome: array[0..1] of TActorAction = (aaGreet, aaWave);
var
  I: Word;
  Action: TActorAction;
begin
  if CompareText(Preview.ActorLibrary, Value) <> 0 then
  begin
    Preview.ActorLibrary := Value;
    ActorLibrary.Caption := Preview.ActorLibrary;
    if Preview.ActorVersion <> EmptyStr then
      ActorName.Caption := Preview.ActorName + ' v' + Preview.ActorVersion 
    else
      ActorName.Caption := Preview.ActorName;
    ActorCopyright.Text := Preview.ActorCopyright;
    if Preview.AnimationCount > 0 then
    begin
      Screen.Cursor := crHourglass;
      try
        Animations.RowCount := Preview.AnimationCount + 1;
        for I := 1 to Preview.AnimationCount do
        begin
          with Preview.Animations[I] do
          begin
            Animations.Cells[0, I] := Name;
            if ExitType = etAnimation then
              Animations.Cells[1, I] := ReturnName
            else
              Animations.Cells[1, I] := ExitTypeNames[ExitType];
            Animations.Cells[2, I] := BooleanNames[HasSound];
            if Actions = [] then
              Animations.Cells[3, I] := ActorActionNames[aaNone]
            else
            begin
              Animations.Cells[3, I] := EmptyStr;
              for Action := Low(TActorAction) to High(TActorAction) do
                if Action in Actions then
                  if Animations.Cells[3, I] <> EmptyStr then
                    Animations.Cells[3, I] := Animations.Cells[3, I] + ', ' +
                                              ActorActionNames[Action]
                  else
                    Animations.Cells[3, I] := Animations.Cells[3, I] +
                                              ActorActionNames[Action];
            end;
          end;
        end;
        Animations.Row := 1;
        if Preview.Play(Welcome[Random(High(Welcome)+1)]) = 0 then
          Preview.Play(aaRestPose);
      finally
        Screen.Cursor := crDefault;
      end;
    end
    else
    begin
      Animations.RowCount := 2;
      Animations.Cells[0,1] := EmptyStr;
      Animations.Cells[1,1] := EmptyStr;
      Animations.Cells[2,1] := EmptyStr;
      Animations.Cells[3,1] := EmptyStr;
    end;
  end;
end;

procedure TActorLibDialog.FormCreate(Sender: TObject);
begin
  Preview := TAssistantPanel.Create(Self);
  Preview.Parent := Self;
  Preview.AutoSize := False;
  Preview.BoundsRect := Panel.BoundsRect;
  Animations.Cells[0,0] := 'Animation';
  Animations.Cells[1,0] := 'Return Animation';
  Animations.Cells[2,0] := 'Has Sound';
  Animations.Cells[3,0] := 'Assigned to Action';
end;

procedure TActorLibDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  Preview.StopAll;
  CanClose := True;
end;

procedure TActorLibDialog.AnimationsDblClick(Sender: TObject);
begin
  if Animations.Row > 0 then
  begin
    if Preview.Playing then
      Preview.StopAll;
    Preview.Play(Animations.Row);
  end;
end;

procedure TActorLibDialog.AnimationsKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = ' ') and (Animations.Row > 0) then
  begin
    if Preview.Playing then
      Preview.StopAll;
    Preview.Play(Animations.Row);
  end;
end;

procedure TActorLibDialog.BtnLoadClick(Sender: TObject);
begin
  OpenDialog.FileName := LibraryName;
  if OpenDialog.Execute then
    LibraryName := OpenDialog.FileName;
end;

procedure TActorLibDialog.BtnClearClick(Sender: TObject);
begin
  LibraryName := EmptyStr;
end;

function SelectActor(Caption: String; var Filename: String): Boolean;
var
  Dialog: TActorLibDialog;
begin
  Result := False;
  Dialog := TActorLibDialog.Create(Application);
  try
    Dialog.Caption := Caption;
    try
      Dialog.LibraryName := Filename;
    except
      // Ignores exceptions
    end;
    if Dialog.Execute then
    begin
      Filename := Dialog.LibraryName;
      Result := True;
    end;
  finally
    Dialog.Free;
  end;
end;

end.
